@@ -1,10 +1,6 @@
 // Tests that keyboard navigation in the search panel works as designed.
 
-const { SearchTestUtils } = ChromeUtils.import(
-  "resource://testing-common/SearchTestUtils.jsm"
-);
 const searchPopup = document.getElementById("PopupSearchAutoComplete");
-const oneOffsContainer = searchPopup.searchOneOffsContainer;
 
 const kValues = ["foo1", "foo2", "foo3"];
 const kUserValue = "foo";
@@ -12,7 +8,9 @@ const kUserValue = "foo";
 function getOpenSearchItems() {
   let os = [];
 
-  let addEngineList = oneOffsContainer.querySelector(".search-add-engines");
+  let addEngineList = searchPopup.searchOneOffsContainer.querySelector(
+    ".search-add-engines"
+  );
   for (
     let item = addEngineList.firstElementChild;
     item;
@@ -27,36 +25,51 @@ function getOpenSearchItems() {
 let searchbar;
 let textbox;
 
-add_task(async function init() {
+async function checkHeader(engine) {
+  // The header can be updated after getting the engine, so we may have to
+  // wait for it.
+  let header = searchPopup.searchbarEngineName;
+  if (!header.getAttribute("value").includes(engine.name)) {
+    await new Promise(resolve => {
+      let observer = new MutationObserver(() => {
+        observer.disconnect();
+        resolve();
+      });
+      observer.observe(searchPopup.searchbarEngineName, {
+        attributes: true,
+        attributeFilter: ["value"],
+      });
+    });
+  }
+  Assert.ok(
+    header.getAttribute("value").includes(engine.name),
+    "Should have the correct engine name displayed in the header"
+  );
+}
+
+add_setup(async function () {
   searchbar = await gCUITestUtils.addSearchBar();
-  registerCleanupFunction(() => {
-    gCUITestUtils.removeSearchBar();
-  });
   textbox = searchbar.textbox;
 
-  await promiseNewEngine("testEngine.xml");
-
+  await SearchTestUtils.promiseNewSearchEngine({
+    url: getRootDirectory(gTestPath) + "testEngine.xml",
+    setAsDefault: true,
+  });
   // First cleanup the form history in case other tests left things there.
-  await new Promise((resolve, reject) => {
-    info("cleanup the search history");
-    searchbar.FormHistory.update(
-      { op: "remove", fieldname: "searchbar-history" },
-      { handleCompletion: resolve, handleError: reject }
-    );
-  });
+  info("cleanup the search history");
+  await FormHistory.update({ op: "remove", fieldname: "searchbar-history" });
 
-  await new Promise((resolve, reject) => {
-    info("adding search history values: " + kValues);
-    let addOps = kValues.map(value => {
-      return { op: "add", fieldname: "searchbar-history", value };
-    });
-    searchbar.FormHistory.update(addOps, {
-      handleCompletion: resolve,
-      handleError: reject,
-    });
+  info("adding search history values: " + kValues);
+  let addOps = kValues.map(value => {
+    return { op: "add", fieldname: "searchbar-history", value };
   });
+  await FormHistory.update(addOps);
 
   textbox.value = kUserValue;
+
+  registerCleanupFunction(async () => {
+    gCUITestUtils.removeSearchBar();
+  });
 });
 
 add_task(async function test_arrows() {
@@ -95,6 +108,7 @@ add_task(async function test_arrows() {
       kValues[i],
       "the textfield value should be " + kValues[i]
     );
+    await checkHeader(Services.search.defaultEngine);
   }
 
   // Pressing down again should remove suggestion selection and change the text
@@ -109,11 +123,13 @@ add_task(async function test_arrows() {
 
   // now cycle through the one-off items, the first one is already selected.
   for (let i = 0; i < oneOffs.length; ++i) {
+    let oneOffButton = oneOffs[i];
     is(
       textbox.selectedButton,
-      oneOffs[i],
+      oneOffButton,
       "the one-off button #" + (i + 1) + " should be selected"
     );
+    await checkHeader(oneOffButton.engine);
     EventUtils.synthesizeKey("KEY_ArrowDown");
   }
 
@@ -121,11 +137,13 @@ add_task(async function test_arrows() {
     textbox.selectedButton.classList.contains("search-setting-button"),
     "the settings item should be selected"
   );
+  await checkHeader(Services.search.defaultEngine);
   EventUtils.synthesizeKey("KEY_ArrowDown");
 
   // We should now be back to the initial situation.
   is(searchPopup.selectedIndex, -1, "no suggestion should be selected");
   ok(!textbox.selectedButton, "no one-off button should be selected");
+  await checkHeader(Services.search.defaultEngine);
 
   info("now test the up arrow key");
   EventUtils.synthesizeKey("KEY_ArrowUp");
@@ -133,15 +151,18 @@ add_task(async function test_arrows() {
     textbox.selectedButton.classList.contains("search-setting-button"),
     "the settings item should be selected"
   );
+  await checkHeader(Services.search.defaultEngine);
 
   // cycle through the one-off items, the first one is already selected.
   for (let i = oneOffs.length; i; --i) {
     EventUtils.synthesizeKey("KEY_ArrowUp");
+    let oneOffButton = oneOffs[i - 1];
     is(
       textbox.selectedButton,
-      oneOffs[i - 1],
+      oneOffButton,
       "the one-off button #" + i + " should be selected"
     );
+    await checkHeader(oneOffButton.engine);
   }
 
   // Another press on up should clear the one-off selection and select the
@@ -160,6 +181,7 @@ add_task(async function test_arrows() {
       kValues[i],
       "the textfield value should be " + kValues[i]
     );
+    await checkHeader(Services.search.defaultEngine);
     EventUtils.synthesizeKey("KEY_ArrowUp");
   }
 
@@ -174,7 +196,7 @@ add_task(async function test_arrows() {
 add_task(async function test_typing_clears_button_selection() {
   is(
     Services.focus.focusedElement,
-    textbox.inputField,
+    textbox,
     "the search bar should be focused"
   ); // from the previous test.
   ok(!textbox.selectedButton, "no button should be selected");
@@ -196,7 +218,7 @@ add_task(async function test_typing_clears_button_selection() {
 add_task(async function test_tab() {
   is(
     Services.focus.focusedElement,
-    textbox.inputField,
+    textbox,
     "the search bar should be focused"
   ); // from the previous test.
 
@@ -336,6 +358,10 @@ add_task(async function test_alt_up() {
   let promise = promiseEvent(searchPopup, "popuphidden");
   EventUtils.synthesizeKey("KEY_Escape");
   await promise;
+  ok(
+    !textbox.selectedButton,
+    "no one-off button should be selected after closing the panel"
+  );
 
   // check that alt+up opens the panel...
   promise = promiseEvent(searchPopup, "popupshown");
@@ -543,8 +569,10 @@ add_task(async function test_open_search() {
   searchbar.focus();
   await promise;
 
-  let engines = getOpenSearchItems();
-  is(engines.length, 2, "the opensearch.html page exposes 2 engines");
+  let engines = searchPopup.querySelectorAll(
+    ".searchbar-engine-one-off-add-engine"
+  );
+  is(engines.length, 3, "the opensearch.html page exposes 3 engines");
 
   // Check that there's initially no selection.
   is(searchPopup.selectedIndex, -1, "no suggestion should be selected");
@@ -567,16 +595,17 @@ add_task(async function test_open_search() {
       "the engine #" + i + " should be selected"
     );
     ok(
-      selectedButton.classList.contains("addengine-item"),
-      "the button is themed as an engine item"
+      selectedButton.classList.contains("searchbar-engine-one-off-add-engine"),
+      "the button is themed as an add engine"
     );
   }
 
   // Pressing up again should select the last one-off button.
   EventUtils.synthesizeKey("KEY_ArrowUp");
+  const allOneOffs = getOneOffs();
   is(
     textbox.selectedButton,
-    getOneOffs().pop(),
+    allOneOffs[allOneOffs.length - engines.length - 1],
     "the last one-off button should be selected"
   );
 
@@ -609,7 +638,7 @@ add_task(async function cleanup() {
   let removeOps = kValues.map(value => {
     return { op: "remove", fieldname: "searchbar-history", value };
   });
-  searchbar.FormHistory.update(removeOps);
+  await FormHistory.update(removeOps);
 
   textbox.value = "";
 });
