@@ -1,3 +1,7 @@
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this file,
+ * You can obtain one at http://mozilla.org/MPL/2.0/. */
+
 "use strict";
 
 this.backgroundPage = class extends ExtensionAPI {
@@ -20,7 +24,11 @@ this.backgroundPage = class extends ExtensionAPI {
 
       runtime: {
         getBackgroundPage() {
-          return context.cloneScope.Promise.resolve(getBackgroundPage());
+          return context.childManager
+            .callParentAsyncFunction("runtime.internalWakeupBackground", [])
+            .then(() => {
+              return context.cloneScope.Promise.resolve(getBackgroundPage());
+            });
         },
       },
     };
