@@ -9,19 +9,9 @@ server.registerPathHandler("/dummyFrame", (request, response) => {
   response.write("");
 });
 
-add_task(async function connect_from_background_frame() {
+add_task(async function content_script_in_background_frame() {
   async function background() {
     const FRAME_URL = "http://example.com:8888/dummyFrame";
-    browser.runtime.onConnect.addListener(port => {
-      browser.test.assertEq(port.sender.tab, undefined, "Sender is not a tab");
-      browser.test.assertEq(port.sender.url, FRAME_URL, "Expected sender URL");
-      port.onMessage.addListener(msg => {
-        browser.test.assertEq("pong", msg, "Reply from content script");
-        port.disconnect();
-      });
-      port.postMessage("ping");
-    });
-
     await browser.contentScripts.register({
       matches: ["http://example.com/dummyFrame"],
       js: [{ file: "contentscript.js" }],
@@ -35,15 +25,7 @@ add_task(async function connect_from_background_frame() {
 
   function contentScript() {
     browser.test.log(`Running content script at ${document.URL}`);
-
-    let port = browser.runtime.connect();
-    port.onMessage.addListener(msg => {
-      browser.test.assertEq("ping", msg, "Expected message to content script");
-      port.postMessage("pong");
-    });
-    port.onDisconnect.addListener(() => {
-      browser.test.sendMessage("disconnected_in_content_script");
-    });
+    browser.test.sendMessage("done_in_content_script");
   }
 
   let extension = ExtensionTestUtils.loadExtension({
@@ -56,6 +38,6 @@ add_task(async function connect_from_background_frame() {
     background,
   });
   await extension.startup();
-  await extension.awaitMessage("disconnected_in_content_script");
+  await extension.awaitMessage("done_in_content_script");
   await extension.unload();
 });
