@@ -10,12 +10,10 @@
 // else we could use nsRange.h and nsIFind.h.
 #include "nsFind.h"
 
-#include "nsIComponentManager.h"
-#include "nsIScriptSecurityManager.h"
+#include "mozilla/dom/ScriptSettings.h"
 #include "nsIInterfaceRequestor.h"
 #include "nsIInterfaceRequestorUtils.h"
 #include "nsPIDOMWindow.h"
-#include "nsIURI.h"
 #include "nsIDocShell.h"
 #include "nsPresContext.h"
 #include "mozilla/dom/Document.h"
@@ -25,7 +23,6 @@
 #include "nsReadableUtils.h"
 #include "nsIContent.h"
 #include "nsContentCID.h"
-#include "nsIServiceManager.h"
 #include "nsIObserverService.h"
 #include "nsISupportsPrimitives.h"
 #include "nsFind.h"
@@ -37,6 +34,7 @@
 #include "mozilla/dom/Element.h"
 #include "mozilla/dom/Selection.h"
 #include "nsISimpleEnumerator.h"
+#include "nsComponentManagerUtils.h"
 #include "nsContentUtils.h"
 #include "nsGenericHTMLElement.h"
 
@@ -45,6 +43,7 @@
 #  include "nsString.h"
 #endif
 
+using namespace mozilla;
 using mozilla::dom::Document;
 using mozilla::dom::Element;
 using mozilla::dom::Selection;
@@ -54,10 +53,11 @@ nsWebBrowserFind::nsWebBrowserFind()
       mWrapFind(false),
       mEntireWord(false),
       mMatchCase(false),
+      mMatchDiacritics(false),
       mSearchSubFrames(true),
       mSearchParentFrames(true) {}
 
-nsWebBrowserFind::~nsWebBrowserFind() {}
+nsWebBrowserFind::~nsWebBrowserFind() = default;
 
 NS_IMPL_ISUPPORTS(nsWebBrowserFind, nsIWebBrowserFind,
                   nsIWebBrowserFindInFrames)
@@ -127,10 +127,9 @@ nsWebBrowserFind::FindNext(bool* aResult) {
   auto enumDirection = mFindBackwards ? nsIDocShell::ENUMERATE_BACKWARDS
                                       : nsIDocShell::ENUMERATE_FORWARDS;
 
-  nsCOMPtr<nsISimpleEnumerator> docShellEnumerator;
-  rv = rootDocShell->GetDocShellEnumerator(nsIDocShellTreeItem::typeAll,
-                                           enumDirection,
-                                           getter_AddRefs(docShellEnumerator));
+  nsTArray<RefPtr<nsIDocShell>> docShells;
+  rv = rootDocShell->GetAllDocShellsInSubtree(nsIDocShellTreeItem::typeAll,
+                                              enumDirection, docShells);
   if (NS_FAILED(rv)) {
     return rv;
   }
@@ -138,23 +137,10 @@ nsWebBrowserFind::FindNext(bool* aResult) {
   // remember where we started
   nsCOMPtr<nsIDocShellTreeItem> startingItem = searchFrame->GetDocShell();
 
-  nsCOMPtr<nsIDocShellTreeItem> curItem;
-
   // XXX We should avoid searching in frameset documents here.
   // We also need to honour mSearchSubFrames and mSearchParentFrames.
-  bool hasMore, doFind = false;
-  while (NS_SUCCEEDED(docShellEnumerator->HasMoreElements(&hasMore)) &&
-         hasMore) {
-    nsCOMPtr<nsISupports> curSupports;
-    rv = docShellEnumerator->GetNext(getter_AddRefs(curSupports));
-    if (NS_FAILED(rv)) {
-      break;
-    }
-    curItem = do_QueryInterface(curSupports, &rv);
-    if (NS_FAILED(rv)) {
-      break;
-    }
-
+  bool doFind = false;
+  for (const auto& curItem : docShells) {
     if (doFind) {
       searchFrame = curItem->GetWindow();
       if (!searchFrame) {
@@ -190,28 +176,15 @@ nsWebBrowserFind::FindNext(bool* aResult) {
   // From here on, we're wrapping, first through the other frames, then finally
   // from the beginning of the starting frame back to the starting point.
 
-  // because nsISimpleEnumerator is totally lame and isn't resettable, I have to
+  // because nsISimpleEnumerator is bad and isn't resettable, I have to
   // make a new one
-  docShellEnumerator = nullptr;
-  rv = rootDocShell->GetDocShellEnumerator(nsIDocShellTreeItem::typeAll,
-                                           enumDirection,
-                                           getter_AddRefs(docShellEnumerator));
+  rv = rootDocShell->GetAllDocShellsInSubtree(nsIDocShellTreeItem::typeAll,
+                                              enumDirection, docShells);
   if (NS_FAILED(rv)) {
     return rv;
   }
 
-  while (NS_SUCCEEDED(docShellEnumerator->HasMoreElements(&hasMore)) &&
-         hasMore) {
-    nsCOMPtr<nsISupports> curSupports;
-    rv = docShellEnumerator->GetNext(getter_AddRefs(curSupports));
-    if (NS_FAILED(rv)) {
-      break;
-    }
-    curItem = do_QueryInterface(curSupports, &rv);
-    if (NS_FAILED(rv)) {
-      break;
-    }
-
+  for (const auto& curItem : docShells) {
     searchFrame = curItem->GetWindow();
     if (!searchFrame) {
       rv = NS_ERROR_FAILURE;
@@ -317,6 +290,19 @@ nsWebBrowserFind::SetMatchCase(bool aMatchCase) {
   return NS_OK;
 }
 
+NS_IMETHODIMP
+nsWebBrowserFind::GetMatchDiacritics(bool* aMatchDiacritics) {
+  NS_ENSURE_ARG_POINTER(aMatchDiacritics);
+  *aMatchDiacritics = mMatchDiacritics;
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsWebBrowserFind::SetMatchDiacritics(bool aMatchDiacritics) {
+  mMatchDiacritics = aMatchDiacritics;
+  return NS_OK;
+}
+
 void nsWebBrowserFind::SetSelectionAndScroll(nsPIDOMWindowOuter* aWindow,
                                              nsRange* aRange) {
   RefPtr<Document> doc = aWindow->GetDoc();
@@ -358,10 +344,10 @@ void nsWebBrowserFind::SetSelectionAndScroll(nsPIDOMWindowOuter* aWindow,
       selCon->GetSelection(nsISelectionController::SELECTION_NORMAL);
   if (selection) {
     selection->RemoveAllRanges(IgnoreErrors());
-    selection->AddRange(*aRange, IgnoreErrors());
+    selection->AddRangeAndSelectFramesAndNotifyListeners(*aRange,
+                                                         IgnoreErrors());
 
-    nsCOMPtr<nsIFocusManager> fm = do_GetService(FOCUSMANAGER_CONTRACTID);
-    if (fm) {
+    if (RefPtr<nsFocusManager> fm = nsFocusManager::GetFocusManager()) {
       if (tcFrame) {
         RefPtr<Element> newFocusedElement = Element::FromNode(content);
         fm->SetFocus(newFocusedElement, nsIFocusManager::FLAG_NOSCROLL);
@@ -444,8 +430,8 @@ nsresult nsWebBrowserFind::GetSearchLimits(nsRange* aSearchRange,
   NS_ENSURE_ARG_POINTER(aSel);
 
   // There is a selection.
-  uint32_t count = aSel->RangeCount();
-  if (count < 1) {
+  const uint32_t rangeCount = aSel->RangeCount();
+  if (rangeCount < 1) {
     return SetRangeAroundDocument(aSearchRange, aStartPt, aEndPt, aDoc);
   }
 
@@ -460,15 +446,20 @@ nsresult nsWebBrowserFind::GetSearchLimits(nsRange* aSearchRange,
   // There are four possible range endpoints we might use:
   // DocumentStart, SelectionStart, SelectionEnd, DocumentEnd.
 
-  RefPtr<nsRange> range;
+  RefPtr<const nsRange> range;
   nsCOMPtr<nsINode> node;
   uint32_t offset;
 
+  // Prevent the security checks in nsRange from getting into effect for the
+  // purposes of determining the search range. These ranges will never be
+  // exposed to content.
+  mozilla::dom::AutoNoJSAPI nojsapi;
+
   // Forward, not wrapping: SelEnd to DocEnd
   if (!mFindBackwards && !aWrap) {
     // This isn't quite right, since the selection's ranges aren't
     // necessarily in order; but they usually will be.
-    range = aSel->GetRangeAt(count - 1);
+    range = aSel->GetRangeAt(rangeCount - 1);
     if (!range) {
       return NS_ERROR_UNEXPECTED;
     }
@@ -506,7 +497,7 @@ nsresult nsWebBrowserFind::GetSearchLimits(nsRange* aSearchRange,
   }
   // Forward, wrapping: DocStart to SelEnd
   else if (!mFindBackwards && aWrap) {
-    range = aSel->GetRangeAt(count - 1);
+    range = aSel->GetRangeAt(rangeCount - 1);
     if (!range) {
       return NS_ERROR_UNEXPECTED;
     }
@@ -652,6 +643,7 @@ nsresult nsWebBrowserFind::SearchInFrame(nsPIDOMWindowOuter* aWindow,
   NS_ENSURE_SUCCESS(rv, rv);
 
   (void)find->SetCaseSensitive(mMatchCase);
+  (void)find->SetMatchDiacritics(mMatchDiacritics);
   (void)find->SetFindBackwards(mFindBackwards);
 
   (void)find->SetEntireWord(mEntireWord);
@@ -663,10 +655,9 @@ nsresult nsWebBrowserFind::SearchInFrame(nsPIDOMWindowOuter* aWindow,
   RefPtr<Selection> sel = GetFrameSelection(aWindow);
   NS_ENSURE_ARG_POINTER(sel);
 
-  RefPtr<nsRange> searchRange = new nsRange(theDoc);
-  RefPtr<nsRange> startPt = new nsRange(theDoc);
-  RefPtr<nsRange> endPt = new nsRange(theDoc);
-  NS_ENSURE_ARG_POINTER(endPt);
+  RefPtr<nsRange> searchRange = nsRange::Create(theDoc);
+  RefPtr<nsRange> startPt = nsRange::Create(theDoc);
+  RefPtr<nsRange> endPt = nsRange::Create(theDoc);
 
   RefPtr<nsRange> foundRange;
 
@@ -758,13 +749,12 @@ nsresult nsWebBrowserFind::OnFind(nsPIDOMWindowOuter* aFoundWindow) {
     ClearFrameSelection(lastFocusedWindow);
   }
 
-  nsCOMPtr<nsIFocusManager> fm = do_GetService(FOCUSMANAGER_CONTRACTID);
-  if (fm) {
+  if (RefPtr<nsFocusManager> fm = nsFocusManager::GetFocusManager()) {
     // get the containing frame and focus it. For top-level windows, the right
     // window should already be focused.
-    RefPtr<Element> frameElement = aFoundWindow->GetFrameElementInternal();
-    if (frameElement) {
-      fm->SetFocus(frameElement, nsIFocusManager::FLAG_BYELEMENTFOCUS);
+    if (RefPtr<Element> frameElement =
+            aFoundWindow->GetFrameElementInternal()) {
+      fm->SetFocus(frameElement, 0);
     }
 
     mLastFocusedWindow = do_GetWeakReference(aFoundWindow);
