@@ -880,15 +880,18 @@ add_task(async function addRemoveBookmarks() {
       })
     );
   }
+  await PlacesFrecencyRecalculator.recalculateAnyOutdatedFrecencies();
   await checkDB([
     ["http://", "example.com", ["http://example.com/"]],
     ["http://", "www.example.com", ["http://www.example.com/"]],
   ]);
   await PlacesUtils.bookmarks.remove(bookmarks[0]);
   await PlacesUtils.history.clear();
+  await PlacesFrecencyRecalculator.recalculateAnyOutdatedFrecencies();
   await checkDB([["http://", "www.example.com", ["http://www.example.com/"]]]);
   await PlacesUtils.bookmarks.remove(bookmarks[1]);
   await PlacesUtils.history.clear();
+  await PlacesFrecencyRecalculator.recalculateAnyOutdatedFrecencies();
   await checkDB([]);
   await cleanUp();
 });
@@ -905,6 +908,7 @@ add_task(async function changeBookmarks() {
       })
     );
   }
+  await PlacesFrecencyRecalculator.recalculateAnyOutdatedFrecencies();
   await checkDB([
     ["http://", "example.com", ["http://example.com/"]],
     ["http://", "www.example.com", ["http://www.example.com/"]],
@@ -914,6 +918,7 @@ add_task(async function changeBookmarks() {
     guid: bookmarks[0].guid,
   });
   await PlacesUtils.history.clear();
+  await PlacesFrecencyRecalculator.recalculateAnyOutdatedFrecencies();
   await checkDB([["http://", "www.example.com", ["http://www.example.com/"]]]);
   await cleanUp();
 });
@@ -973,7 +978,7 @@ add_task(async function moreOriginFrecencyStats() {
     title: "A bookmark",
     url: NetUtil.newURI("http://example.com/1"),
   });
-
+  await PlacesFrecencyRecalculator.recalculateAnyOutdatedFrecencies();
   await checkDB([
     [
       "http://",
@@ -997,6 +1002,7 @@ add_task(async function moreOriginFrecencyStats() {
   // contributing to the frecency stats.
   await PlacesUtils.bookmarks.remove(bookmark);
   await PlacesUtils.history.remove("http://example.com/1");
+  await PlacesFrecencyRecalculator.recalculateAnyOutdatedFrecencies();
   await checkDB([["http://", "example.com", ["http://example.com/0"]]]);
 
   // Remove URL 0.
@@ -1014,8 +1020,16 @@ add_task(async function moreOriginFrecencyStats() {
  *         An array of URL strings.
  * @return The expected origin frecency.
  */
-function expectedOriginFrecency(urls) {
-  return urls.reduce((sum, url) => sum + Math.max(frecencyForUrl(url), 0), 0);
+async function expectedOriginFrecency(urls) {
+  let value = 0;
+  for (let url of urls) {
+    let v = Math.max(
+      await PlacesTestUtils.getDatabaseValue("moz_places", "frecency", { url }),
+      0
+    );
+    value += v;
+  }
+  return value;
 }
 
 /**
@@ -1049,14 +1063,17 @@ async function checkDB(expectedOrigins) {
     }
     return o;
   });
-  expectedOrigins = expectedOrigins.map(o => {
-    return o
-      .slice(0, 2)
-      .concat(checkFrecencies ? expectedOriginFrecency(o[2]) : []);
-  });
-  Assert.deepEqual(actualOrigins, expectedOrigins);
+  let expected = [];
+  for (let origin of expectedOrigins) {
+    expected.push(
+      origin
+        .slice(0, 2)
+        .concat(checkFrecencies ? await expectedOriginFrecency(origin[2]) : [])
+    );
+  }
+  Assert.deepEqual(actualOrigins, expected);
   if (checkFrecencies) {
-    await checkStats(expectedOrigins.map(o => o[2]).filter(o => o > 0));
+    await checkStats(expected.map(o => o[2]).filter(o => o > 0));
   }
 }
 
@@ -1088,9 +1105,9 @@ async function promiseStats() {
   let db = await PlacesUtils.promiseDBConnection();
   let rows = await db.execute(`
     SELECT
-      IFNULL((SELECT value FROM moz_meta WHERE key = "origin_frecency_count"), 0),
-      IFNULL((SELECT value FROM moz_meta WHERE key = "origin_frecency_sum"), 0),
-      IFNULL((SELECT value FROM moz_meta WHERE key = "origin_frecency_sum_of_squares"), 0)
+      IFNULL((SELECT value FROM moz_meta WHERE key = 'origin_frecency_count'), 0),
+      IFNULL((SELECT value FROM moz_meta WHERE key = 'origin_frecency_sum'), 0),
+      IFNULL((SELECT value FROM moz_meta WHERE key = 'origin_frecency_sum_of_squares'), 0)
   `);
   return {
     count: rows[0].getResultByIndex(0),
