@@ -5,7 +5,9 @@
 var testnum = 0;
 var dbConnection; // used for deleted table tests
 
-const { Promise } = ChromeUtils.import("resource://gre/modules/Promise.jsm");
+const { PromiseUtils } = ChromeUtils.importESModule(
+  "resource://gre/modules/PromiseUtils.sys.mjs"
+);
 
 function countDeletedEntries(expected) {
   return new Promise((resolve, reject) => {
@@ -64,7 +66,7 @@ function promiseUpdateEntry(op, name, value) {
   return promiseUpdate(change);
 }
 
-add_task(async function() {
+add_task(async function () {
   let oldSupportsDeletedTable = FormHistory._supportsDeletedTable;
   FormHistory._supportsDeletedTable = true;
 
@@ -108,7 +110,7 @@ add_task(async function() {
     dbFile.append("formhistory.sqlite");
     dbConnection = Services.storage.openUnsharedDatabase(dbFile);
 
-    let deferred = Promise.defer();
+    let deferred = PromiseUtils.defer();
 
     let stmt = dbConnection.createAsyncStatement(
       "DELETE FROM moz_deleted_formhistory"
@@ -140,22 +142,8 @@ add_task(async function() {
     // Cannot use promiseCountEntries when name and value are null
     // because it treats null values as not set
     // and here a search should be done explicity for null.
-    deferred = Promise.defer();
-    await FormHistory.count(
-      { fieldname: null, value: null },
-      {
-        handleResult: result => checkNotExists(result),
-        handleError(error) {
-          do_throw("Error occurred searching form history: " + error);
-        },
-        handleCompletion(reason) {
-          if (!reason) {
-            deferred.resolve();
-          }
-        },
-      }
-    );
-    await deferred.promise;
+    let count = await FormHistory.count({ fieldname: null, value: null });
+    checkNotExists(count);
 
     // ===== 3 =====
     // Test removeEntriesForName with a single matching value
@@ -248,7 +236,7 @@ add_task(async function() {
 
     let processFirstResult = function processResults(results) {
       // Only handle the first result
-      if (results.length > 0) {
+      if (results.length) {
         let result = results[0];
         return [
           result.timesUsed,
@@ -482,16 +470,12 @@ add_task(async function() {
     // Cannot use arrow functions, see bug 1237961.
     await Assert.rejects(
       promiseUpdate({ op: "bump", fieldname: "field5", value: "value5" }),
-      function(err) {
-        return err.result == Ci.mozIStorageError.MISUSE;
-      },
+      /Form history is disabled, only remove operations are allowed/,
       "bumping when form history is disabled should fail"
     );
     await Assert.rejects(
       promiseUpdate({ op: "add", fieldname: "field5", value: "value5" }),
-      function(err) {
-        return err.result == Ci.mozIStorageError.MISUSE;
-      },
+      /Form history is disabled, only remove operations are allowed/,
       "Adding when form history is disabled should fail"
     );
     await Assert.rejects(
@@ -499,9 +483,7 @@ add_task(async function() {
         { op: "update", fieldname: "field5", value: "value5" },
         { op: "remove", fieldname: "field5", value: "value5" },
       ]),
-      function(err) {
-        return err.result == Ci.mozIStorageError.MISUSE;
-      },
+      /Form history is disabled, only remove operations are allowed/,
       "mixed operations when form history is disabled should fail"
     );
     await Assert.rejects(
@@ -513,9 +495,7 @@ add_task(async function() {
         {},
         { op: "remove", fieldname: "field5", value: "value5" },
       ]),
-      function(err) {
-        return err.result == Ci.mozIStorageError.MISUSE;
-      },
+      /Form history is disabled, only remove operations are allowed/,
       "Invalid entries when form history is disabled should fail"
     );
 
