@@ -8,34 +8,51 @@
  * checked in the second request.
  */
 
-const { CommonUtils } = ChromeUtils.import(
-  "resource://services-common/utils.js"
+const { ClientID } = ChromeUtils.importESModule(
+  "resource://gre/modules/ClientID.sys.mjs"
 );
-const { ClientID } = ChromeUtils.import("resource://gre/modules/ClientID.jsm");
-const { Services } = ChromeUtils.import("resource://gre/modules/Services.jsm");
-ChromeUtils.import("resource://gre/modules/XPCOMUtils.jsm", this);
-ChromeUtils.import("resource://gre/modules/TelemetryController.jsm", this);
-ChromeUtils.import("resource://gre/modules/TelemetryStorage.jsm", this);
-ChromeUtils.import("resource://gre/modules/TelemetrySend.jsm", this);
-ChromeUtils.import("resource://gre/modules/TelemetryArchive.jsm", this);
-ChromeUtils.import("resource://gre/modules/TelemetryUtils.jsm", this);
-const { Preferences } = ChromeUtils.import(
-  "resource://gre/modules/Preferences.jsm"
+const { TelemetryController } = ChromeUtils.importESModule(
+  "resource://gre/modules/TelemetryController.sys.mjs"
 );
-ChromeUtils.import("resource://testing-common/ContentTaskUtils.jsm", this);
+const { TelemetryStorage } = ChromeUtils.importESModule(
+  "resource://gre/modules/TelemetryStorage.sys.mjs"
+);
+const { TelemetrySend } = ChromeUtils.importESModule(
+  "resource://gre/modules/TelemetrySend.sys.mjs"
+);
+const { TelemetryArchive } = ChromeUtils.importESModule(
+  "resource://gre/modules/TelemetryArchive.sys.mjs"
+);
+const { TelemetryUtils } = ChromeUtils.importESModule(
+  "resource://gre/modules/TelemetryUtils.sys.mjs"
+);
+const { Preferences } = ChromeUtils.importESModule(
+  "resource://gre/modules/Preferences.sys.mjs"
+);
+const { ContentTaskUtils } = ChromeUtils.importESModule(
+  "resource://testing-common/ContentTaskUtils.sys.mjs"
+);
+const { TestUtils } = ChromeUtils.importESModule(
+  "resource://testing-common/TestUtils.sys.mjs"
+);
+const { TelemetryArchiveTesting } = ChromeUtils.importESModule(
+  "resource://testing-common/TelemetryArchiveTesting.sys.mjs"
+);
+
+ChromeUtils.defineESModuleGetters(this, {
+  JsonSchemaValidator:
+    "resource://gre/modules/components-utils/JsonSchemaValidator.sys.mjs",
+  jwcrypto: "resource://services-crypto/jwcrypto.sys.mjs",
+});
 
 const PING_FORMAT_VERSION = 4;
-const OPTOUT_PING_TYPE = "optout";
+const DELETION_REQUEST_PING_TYPE = "deletion-request";
 const TEST_PING_TYPE = "test-ping-type";
 
-const PLATFORM_VERSION = "1.9.2";
-const APP_VERSION = "1";
-const APP_NAME = "XPCShell";
-
 var gClientID = null;
 
-XPCOMUtils.defineLazyGetter(this, "DATAREPORTING_PATH", function() {
-  return OS.Path.join(OS.Constants.Path.profileDir, "datareporting");
+XPCOMUtils.defineLazyGetter(this, "DATAREPORTING_PATH", async function () {
+  return PathUtils.join(PathUtils.profileDir, "datareporting");
 });
 
 function sendPing(aSendClientId, aSendEnvironment) {
@@ -112,7 +129,12 @@ function checkPingFormat(aPing, aType, aHasClientId, aHasEnvironment) {
 add_task(async function test_setup() {
   // Addon manager needs a profile directory
   do_get_profile();
-  loadAddonManager("xpcshell@tests.mozilla.org", "XPCShell", "1", "1.9.2");
+  await loadAddonManager(
+    "xpcshell@tests.mozilla.org",
+    "XPCShell",
+    "1",
+    "1.9.2"
+  );
   finishAddonManagerStartup();
   fakeIntlReady();
   // Make sure we don't generate unexpected pings due to pref changes.
@@ -159,13 +181,13 @@ add_task(async function test_disableDataUpload() {
   const OPTIN_PROBE = "telemetry.data_upload_optin";
   const isUnified = Preferences.get(TelemetryUtils.Preferences.Unified, false);
   if (!isUnified) {
-    // Skipping the test if unified telemetry is off, as no optout ping will
-    // be generated.
+    // Skipping the test if unified telemetry is off, as no deletion-request ping will be generated.
     return;
   }
 
-  // Check that the optin probe is not set, there should be other data in the snapshot though
-  let snapshot = Telemetry.getSnapshotForScalars("main", false).parent;
+  // Check that the optin probe is not set.
+  // (If there are no recorded scalars, "parent" will be undefined).
+  let snapshot = Telemetry.getSnapshotForScalars("main", false).parent || {};
   Assert.ok(
     !(OPTIN_PROBE in snapshot),
     "Data optin scalar should not be set at start"
@@ -176,6 +198,7 @@ add_task(async function test_disableDataUpload() {
   let ping = await PingServer.promiseNextPing();
   checkPingFormat(ping, TEST_PING_TYPE, true, false);
   let firstClientId = ping.clientId;
+
   Assert.ok(firstClientId, "Test ping needs a client ID");
   Assert.notEqual(
     TelemetryUtils.knownClientID,
@@ -183,18 +206,26 @@ add_task(async function test_disableDataUpload() {
     "Client ID should be valid and random"
   );
 
-  // Disable FHR upload: this should trigger a optout ping.
+  // The next step should trigger an event, watch for it.
+  let disableObserved = TestUtils.topicObserved(
+    TelemetryUtils.TELEMETRY_UPLOAD_DISABLED_TOPIC
+  );
+
+  // Disable FHR upload: this should trigger a deletion-request ping.
   Preferences.set(TelemetryUtils.Preferences.FhrUploadEnabled, false);
 
+  // Wait for the disable event
+  await disableObserved;
+
   ping = await PingServer.promiseNextPing();
-  checkPingFormat(ping, OPTOUT_PING_TYPE, false, false);
+  checkPingFormat(ping, DELETION_REQUEST_PING_TYPE, true, false);
   // Wait on ping activity to settle.
   await TelemetrySend.testWaitOnOutgoingPings();
 
-  snapshot = Telemetry.getSnapshotForScalars("main", false).parent;
+  snapshot = Telemetry.getSnapshotForScalars("main", false).parent || {};
   Assert.ok(
     !(OPTIN_PROBE in snapshot),
-    "Data optin scalar should not be set after optout"
+    "Data optin scalar should not be set after opt out"
   );
 
   // Restore FHR Upload.
@@ -209,20 +240,29 @@ add_task(async function test_disableDataUpload() {
     );
   });
 
-  snapshot = Telemetry.getSnapshotForScalars("main", false).parent;
+  snapshot = Telemetry.getSnapshotForScalars("main", false).parent || {};
   Assert.ok(
     snapshot[OPTIN_PROBE],
     "Enabling data upload should set optin probe"
   );
 
-  // Simulate a failure in sending the optout ping by disabling the HTTP server.
+  // The clientId should've been reset when we restored FHR Upload.
+  let secondClientId = TelemetryController.getCurrentPingData().clientId;
+  Assert.notEqual(
+    firstClientId,
+    secondClientId,
+    "The client id must have changed"
+  );
+  // Simulate a failure in sending the deletion-request ping by disabling the HTTP server.
   await PingServer.stop();
 
   // Try to send a ping. It will be saved as pending  and get deleted when disabling upload.
   TelemetryController.submitExternalPing(TEST_PING_TYPE, {});
 
-  // Disable FHR upload to send a optout ping again.
+  // Disable FHR upload to send a deletion-request ping again.
   Preferences.set(TelemetryUtils.Preferences.FhrUploadEnabled, false);
+  // Wait for the deletion-request ping to be submitted.
+  await TelemetryController.testPromiseDeletionRequestPingSubmitted();
 
   // Wait on sending activity to settle, as |TelemetryController.testReset()| doesn't do that.
   await TelemetrySend.testWaitOnOutgoingPings();
@@ -236,8 +276,8 @@ add_task(async function test_disableDataUpload() {
   let pendingPings = await TelemetryStorage.loadPendingPingList();
   Assert.equal(
     pendingPings.length,
-    0,
-    "All the pending pings should have been deleted, including the optout ping"
+    1,
+    "All the pending pings should have been deleted, except the deletion-request ping"
   );
 
   // Enable the ping server again.
@@ -260,7 +300,7 @@ add_task(async function test_disableDataUpload() {
   // Send a test ping
   await sendPing(true, false);
 
-  // We should have only received the test ping
+  // We should have received the test ping first.
   ping = await PingServer.promiseNextPing();
   checkPingFormat(ping, TEST_PING_TYPE, true, false);
 
@@ -276,6 +316,15 @@ add_task(async function test_disableDataUpload() {
     "Client ID should be different from the previous value"
   );
 
+  // The "deletion-request" ping should come next, as it was pending.
+  ping = await PingServer.promiseNextPing();
+  checkPingFormat(ping, DELETION_REQUEST_PING_TYPE, true, false);
+  Assert.equal(
+    secondClientId,
+    ping.clientId,
+    "Deletion must be requested for correct client id"
+  );
+
   // Wait on ping activity to settle before moving on to the next test. If we were
   // to shut down telemetry, even though the PingServer caught the expected pings,
   // TelemetrySend could still be processing them (clearing pings would happen in
@@ -403,11 +452,10 @@ add_task(async function test_archivePings() {
     : TelemetryUtils.Preferences.TelemetryEnabled;
   Preferences.set(uploadPref, false);
 
-  // If we're using unified telemetry, disabling ping upload will generate a "optout"
-  // ping. Catch it.
+  // If we're using unified telemetry, disabling ping upload will generate a "deletion-request" ping. Catch it.
   if (isUnified) {
     let ping = await PingServer.promiseNextPing();
-    checkPingFormat(ping, OPTOUT_PING_TYPE, false, false);
+    checkPingFormat(ping, DELETION_REQUEST_PING_TYPE, true, false);
   }
 
   // Register a new Ping Handler that asserts if a ping is received, then send a ping.
@@ -529,7 +577,10 @@ add_task(async function test_midnightPingSendFuzzing() {
 
   // Clean-up.
   fakeMidnightPingFuzzingDelay(0);
-  fakePingSendTimer(() => {}, () => {});
+  fakePingSendTimer(
+    () => {},
+    () => {}
+  );
 });
 
 add_task(async function test_changePingAfterSubmission() {
@@ -555,78 +606,38 @@ add_task(async function test_changePingAfterSubmission() {
   );
 });
 
-add_task(
-  {
-    skip_if: () =>
-      Services.prefs.getBoolPref(TelemetryUtils.Preferences.Unified, false),
-  },
-  async function test_telemetryEnabledUnexpectedValue() {
-    // Remove the default value for toolkit.telemetry.enabled from the default prefs.
-    // Otherwise, we wouldn't be able to set the pref to a string.
-    let defaultPrefBranch = Services.prefs.getDefaultBranch(null);
-    defaultPrefBranch.deleteBranch(TelemetryUtils.Preferences.TelemetryEnabled);
-
-    // Set the preferences controlling the Telemetry status to a string.
-    Preferences.set(TelemetryUtils.Preferences.TelemetryEnabled, "false");
-    // Check that Telemetry is not enabled.
-    await TelemetryController.testReset();
-    Assert.equal(
-      Telemetry.canRecordExtended,
-      false,
-      "Invalid values must not enable Telemetry recording."
-    );
-
-    // Delete the pref again.
-    defaultPrefBranch.deleteBranch(TelemetryUtils.Preferences.TelemetryEnabled);
-
-    // Make sure that flipping it to true works.
-    Preferences.set(TelemetryUtils.Preferences.TelemetryEnabled, true);
-    await TelemetryController.testReset();
-    Assert.equal(
-      Telemetry.canRecordExtended,
-      true,
-      "True must enable Telemetry recording."
-    );
-
-    // Also check that the false works as well.
-    Preferences.set(TelemetryUtils.Preferences.TelemetryEnabled, false);
-    await TelemetryController.testReset();
-    Assert.equal(
-      Telemetry.canRecordExtended,
-      false,
-      "False must disable Telemetry recording."
-    );
-
-    // Restore the state of the pref.
-    Preferences.set(TelemetryUtils.Preferences.TelemetryEnabled, true);
-  }
-);
-
 add_task(async function test_telemetryCleanFHRDatabase() {
   const FHR_DBNAME_PREF = "datareporting.healthreport.dbName";
   const CUSTOM_DB_NAME = "unlikely.to.be.used.sqlite";
   const DEFAULT_DB_NAME = "healthreport.sqlite";
 
   // Check that we're able to remove a FHR DB with a custom name.
+  const profileDir = PathUtils.profileDir;
   const CUSTOM_DB_PATHS = [
-    OS.Path.join(OS.Constants.Path.profileDir, CUSTOM_DB_NAME),
-    OS.Path.join(OS.Constants.Path.profileDir, CUSTOM_DB_NAME + "-wal"),
-    OS.Path.join(OS.Constants.Path.profileDir, CUSTOM_DB_NAME + "-shm"),
+    PathUtils.join(profileDir, CUSTOM_DB_NAME),
+    PathUtils.join(profileDir, CUSTOM_DB_NAME + "-wal"),
+    PathUtils.join(profileDir, CUSTOM_DB_NAME + "-shm"),
   ];
   Preferences.set(FHR_DBNAME_PREF, CUSTOM_DB_NAME);
 
   // Write fake DB files to the profile directory.
   for (let dbFilePath of CUSTOM_DB_PATHS) {
-    await OS.File.writeAtomic(dbFilePath, "some data");
+    await IOUtils.writeUTF8(dbFilePath, "some data");
   }
 
   // Trigger the cleanup and check that the files were removed.
   await TelemetryStorage.removeFHRDatabase();
   for (let dbFilePath of CUSTOM_DB_PATHS) {
-    Assert.ok(
-      !(await OS.File.exists(dbFilePath)),
-      "The DB must not be on the disk anymore: " + dbFilePath
-    );
+    try {
+      await IOUtils.read(dbFilePath);
+    } catch (e) {
+      Assert.ok(DOMException.isInstance(e));
+      Assert.equal(
+        e.name,
+        "NotFoundError",
+        "The DB must not be on the disk anymore: " + dbFilePath
+      );
+    }
   }
 
   // We should not break anything if there's no DB file.
@@ -636,30 +647,36 @@ add_task(async function test_telemetryCleanFHRDatabase() {
   Preferences.reset(FHR_DBNAME_PREF);
 
   const DEFAULT_DB_PATHS = [
-    OS.Path.join(OS.Constants.Path.profileDir, DEFAULT_DB_NAME),
-    OS.Path.join(OS.Constants.Path.profileDir, DEFAULT_DB_NAME + "-wal"),
-    OS.Path.join(OS.Constants.Path.profileDir, DEFAULT_DB_NAME + "-shm"),
+    PathUtils.join(profileDir, DEFAULT_DB_NAME),
+    PathUtils.join(profileDir, DEFAULT_DB_NAME + "-wal"),
+    PathUtils.join(profileDir, DEFAULT_DB_NAME + "-shm"),
   ];
 
   // Write fake DB files to the profile directory.
   for (let dbFilePath of DEFAULT_DB_PATHS) {
-    await OS.File.writeAtomic(dbFilePath, "some data");
+    await IOUtils.writeUTF8(dbFilePath, "some data");
   }
 
   // Trigger the cleanup and check that the files were removed.
   await TelemetryStorage.removeFHRDatabase();
   for (let dbFilePath of DEFAULT_DB_PATHS) {
-    Assert.ok(
-      !(await OS.File.exists(dbFilePath)),
-      "The DB must not be on the disk anymore: " + dbFilePath
-    );
+    try {
+      await IOUtils.read(dbFilePath);
+    } catch (e) {
+      Assert.ok(DOMException.isInstance(e));
+      Assert.equal(
+        e.name,
+        "NotFoundError",
+        "The DB must not be on the disk anymore: " + dbFilePath
+      );
+    }
   }
 });
 
 add_task(async function test_sendNewProfile() {
   if (
     gIsAndroid ||
-    (AppConstants.platform == "linux" && OS.Constants.Sys.bits == 32)
+    (AppConstants.platform == "linux" && !Services.appinfo.is64Bit)
   ) {
     // We don't support the pingsender on Android, yet, see bug 1335917.
     // We also don't suppor the pingsender testing on Treeherder for
@@ -674,7 +691,7 @@ add_task(async function test_sendNewProfile() {
 
   // Make sure Telemetry is shut down before beginning and that we have
   // no pending pings.
-  let resetTest = async function() {
+  let resetTest = async function () {
     await TelemetryController.testShutdown();
     await TelemetryStorage.testClearPendingPings();
     PingServer.clearRequests();
@@ -682,8 +699,11 @@ add_task(async function test_sendNewProfile() {
   await resetTest();
 
   // Make sure to reset all the new-profile ping prefs.
-  const stateFilePath = OS.Path.join(DATAREPORTING_PATH, "session-state.json");
-  await OS.File.remove(stateFilePath, { ignoreAbsent: true });
+  const stateFilePath = PathUtils.join(
+    await DATAREPORTING_PATH,
+    "session-state.json"
+  );
+  await IOUtils.remove(stateFilePath);
   Preferences.set(PREF_NEWPROFILE_DELAY, 1);
   Preferences.set(PREF_NEWPROFILE_ENABLED, true);
 
@@ -698,6 +718,10 @@ add_task(async function test_sendNewProfile() {
     "startup",
     "The new-profile ping generated after startup must have the correct reason"
   );
+  Assert.ok(
+    "parent" in ping.payload.processes,
+    "The new-profile ping generated after startup must have processes.parent data"
+  );
 
   // Check that is not sent with the pingsender during startup.
   Assert.throws(
@@ -708,7 +732,7 @@ add_task(async function test_sendNewProfile() {
 
   // Make sure that the new-profile ping is sent at shutdown if it wasn't sent before.
   await resetTest();
-  await OS.File.remove(stateFilePath, { ignoreAbsent: true });
+  await IOUtils.remove(stateFilePath);
   Preferences.reset(PREF_NEWPROFILE_DELAY);
 
   nextReq = PingServer.promiseNextRequest();
@@ -722,6 +746,10 @@ add_task(async function test_sendNewProfile() {
     "shutdown",
     "The new-profile ping generated at shutdown must have the correct reason"
   );
+  Assert.ok(
+    "parent" in ping.payload.processes,
+    "The new-profile ping generated at shutdown must have processes.parent data"
+  );
 
   // Check that the new-profile ping is sent at shutdown using the pingsender.
   Assert.equal(
@@ -747,13 +775,13 @@ add_task(async function test_sendNewProfile() {
   // Check that we don't send the new-profile ping if the profile already contains
   // a state file (but no "newProfilePingSent" property).
   await resetTest();
-  await OS.File.remove(stateFilePath, { ignoreAbsent: true });
+  await IOUtils.remove(stateFilePath);
   const sessionState = {
     sessionId: null,
     subsessionId: null,
     profileSubsessionCounter: 3785,
   };
-  await CommonUtils.writeJSON(sessionState, stateFilePath);
+  await IOUtils.writeJSON(stateFilePath, sessionState);
   await TelemetryController.testReset();
   await TelemetryController.testShutdown();
 
@@ -762,6 +790,321 @@ add_task(async function test_sendNewProfile() {
   PingServer.resetPingHandler();
 });
 
+add_task(async function test_encryptedPing() {
+  if (gIsAndroid) {
+    // The underlying jwcrypto module being used here is not currently available on Android.
+    return;
+  }
+  Cu.importGlobalProperties(["crypto"]);
+
+  const ECDH_PARAMS = {
+    name: "ECDH",
+    namedCurve: "P-256",
+  };
+
+  const privateKey = {
+    crv: "P-256",
+    d: "rcs093UlGDG6piwHenmSDoAxbzMIXT43JkQbkt3xEmI",
+    ext: true,
+    key_ops: ["deriveKey"],
+    kty: "EC",
+    x: "h12feyTYBZ__wO_AnM1a5-KTDlko3-YyQ_en19jyrs0",
+    y: "6GSfzo14ehDyH5E-xCOedJDAYlN0AGPMCtIgFbheLko",
+  };
+
+  const publicKey = {
+    crv: "P-256",
+    ext: true,
+    kty: "EC",
+    x: "h12feyTYBZ__wO_AnM1a5-KTDlko3-YyQ_en19jyrs0",
+    y: "6GSfzo14ehDyH5E-xCOedJDAYlN0AGPMCtIgFbheLko",
+  };
+
+  const pioneerId = "12345";
+  const schemaName = "abc";
+  const schemaNamespace = "def";
+  const schemaVersion = 2;
+
+  Services.prefs.setStringPref("toolkit.telemetry.pioneerId", pioneerId);
+
+  // Stop the sending task and then start it again.
+  await TelemetrySend.shutdown();
+  // Reset the controller to spin the ping sending task.
+  await TelemetryController.testReset();
+
+  // Submit a ping with a custom payload, which will be encrypted.
+  let payload = { canary: "test" };
+  let pingPromise = TelemetryController.submitExternalPing(
+    "pioneer-study",
+    payload,
+    {
+      studyName: "pioneer-dev-1@allizom.org",
+      addPioneerId: true,
+      useEncryption: true,
+      encryptionKeyId: "pioneer-dev-20200423",
+      publicKey,
+      schemaName,
+      schemaNamespace,
+      schemaVersion,
+    }
+  );
+
+  // Wait for the ping to be archived.
+  const pingId = await pingPromise;
+
+  let archivedCopy = await TelemetryArchive.promiseArchivedPingById(pingId);
+
+  Assert.notEqual(
+    archivedCopy.payload.encryptedData,
+    payload,
+    "The encrypted payload must not match the plaintext."
+  );
+
+  Assert.equal(
+    archivedCopy.payload.pioneerId,
+    pioneerId,
+    "Pioneer ID in ping must match the pref."
+  );
+
+  // Validate ping against schema.
+  const schema = {
+    $schema: "http://json-schema.org/draft-04/schema#",
+    properties: {
+      application: {
+        additionalProperties: false,
+        properties: {
+          architecture: {
+            type: "string",
+          },
+          buildId: {
+            pattern: "^[0-9]{10}",
+            type: "string",
+          },
+          channel: {
+            type: "string",
+          },
+          displayVersion: {
+            pattern: "^[0-9]{2,3}\\.",
+            type: "string",
+          },
+          name: {
+            type: "string",
+          },
+          platformVersion: {
+            pattern: "^[0-9]{2,3}\\.",
+            type: "string",
+          },
+          vendor: {
+            type: "string",
+          },
+          version: {
+            pattern: "^[0-9]{2,3}\\.",
+            type: "string",
+          },
+          xpcomAbi: {
+            type: "string",
+          },
+        },
+        required: [
+          "architecture",
+          "buildId",
+          "channel",
+          "name",
+          "platformVersion",
+          "version",
+          "vendor",
+          "xpcomAbi",
+        ],
+        type: "object",
+      },
+      creationDate: {
+        pattern:
+          "^[0-9]{4}-[0-9]{2}-[0-9]{2}T[0-9]{2}:[0-9]{2}:[0-9]{2}\\.[0-9]{3}Z$",
+        type: "string",
+      },
+      id: {
+        pattern:
+          "^[a-fA-F0-9]{8}-[a-fA-F0-9]{4}-[a-fA-F0-9]{4}-[a-fA-F0-9]{4}-[a-fA-F0-9]{12}$",
+        type: "string",
+      },
+      payload: {
+        description: "",
+        properties: {
+          encryptedData: {
+            description: "JOSE/JWE encrypted payload.",
+            type: "string",
+          },
+          encryptionKeyId: {
+            description: "JOSE/JWK key id, e.g. pioneer-20170520.",
+            type: "string",
+          },
+          pioneerId: {
+            description: "Custom pioneer id, must not be Telemetry clientId",
+            pattern:
+              "^[a-fA-F0-9]{8}-[a-fA-F0-9]{4}-[a-fA-F0-9]{4}-[a-fA-F0-9]{4}-[a-fA-F0-9]{12}$",
+            type: "string",
+          },
+          schemaName: {
+            description:
+              "Name of a schema used for validation of the encryptedData",
+            maxLength: 100,
+            minLength: 1,
+            pattern: "^\\S+$",
+            type: "string",
+          },
+          schemaNamespace: {
+            description:
+              "The namespace of the schema used for validation and routing to a dataset.",
+            maxLength: 100,
+            minLength: 1,
+            pattern: "^\\S+$",
+            type: "string",
+          },
+          schemaVersion: {
+            description: "Integer version number of the schema",
+            minimum: 1,
+            type: "integer",
+          },
+          studyName: {
+            description: "Name of a particular study. Usually the addon_id.",
+            maxLength: 100,
+            minLength: 1,
+            pattern: "^\\S+$",
+            type: "string",
+          },
+        },
+        required: [
+          "encryptedData",
+          "encryptionKeyId",
+          "pioneerId",
+          "studyName",
+          "schemaName",
+          "schemaNamespace",
+          "schemaVersion",
+        ],
+        title: "pioneer-study",
+        type: "object",
+      },
+      type: {
+        description: "doc_type, restated",
+        enum: ["pioneer-study"],
+        type: "string",
+      },
+      version: {
+        maximum: 4,
+        minimum: 4,
+        type: "integer",
+      },
+    },
+    required: [
+      "application",
+      "creationDate",
+      "id",
+      "payload",
+      "type",
+      "version",
+    ],
+    title: "pioneer-study",
+    type: "object",
+  };
+
+  const result = JsonSchemaValidator.validate(archivedCopy, schema);
+
+  Assert.ok(
+    result.valid,
+    `Archived ping should validate against schema: ${result.error}`
+  );
+
+  // check that payload can be decrypted.
+  const privateJWK = await crypto.subtle.importKey(
+    "jwk",
+    privateKey,
+    ECDH_PARAMS,
+    false,
+    ["deriveKey"]
+  );
+
+  const decryptedJWE = await jwcrypto.decryptJWE(
+    archivedCopy.payload.encryptedData,
+    privateJWK
+  );
+
+  Assert.deepEqual(
+    JSON.parse(new TextDecoder("utf-8").decode(decryptedJWE)),
+    payload,
+    "decrypted payload should match"
+  );
+});
+
+add_task(async function test_encryptedPing_overrideId() {
+  if (gIsAndroid) {
+    // The underlying jwcrypto module being used here is not currently available on Android.
+    return;
+  }
+  Cu.importGlobalProperties(["crypto"]);
+
+  const publicKey = {
+    crv: "P-256",
+    ext: true,
+    kty: "EC",
+    x: "h12feyTYBZ__wO_AnM1a5-KTDlko3-YyQ_en19jyrs0",
+    y: "6GSfzo14ehDyH5E-xCOedJDAYlN0AGPMCtIgFbheLko",
+  };
+
+  const prefPioneerId = "12345";
+  const overriddenPioneerId = "c0ffeeaa-bbbb-abab-baba-eeff0ceeff0c";
+  const schemaName = "abc";
+  const schemaNamespace = "def";
+  const schemaVersion = 2;
+
+  Services.prefs.setStringPref("toolkit.telemetry.pioneerId", prefPioneerId);
+
+  let archiveTester = new TelemetryArchiveTesting.Checker();
+  await archiveTester.promiseInit();
+
+  // Submit a ping with a custom payload, which will be encrypted.
+  let payload = { canary: "test" };
+  let pingPromise = TelemetryController.submitExternalPing(
+    "test-pioneer-study-override",
+    payload,
+    {
+      studyName: "pioneer-dev-1@allizom.org",
+      addPioneerId: true,
+      overridePioneerId: overriddenPioneerId,
+      useEncryption: true,
+      encryptionKeyId: "pioneer-dev-20200423",
+      publicKey,
+      schemaName,
+      schemaNamespace,
+      schemaVersion,
+    }
+  );
+
+  // Wait for the ping to be submitted, to have the ping id to scan the
+  // archive for.
+  const pingId = await pingPromise;
+
+  // And then wait for the ping to be available in the archive.
+  await TestUtils.waitForCondition(
+    () => archiveTester.promiseFindPing("test-pioneer-study-override", []),
+    "Failed to find the pioneer ping"
+  );
+
+  let archivedCopy = await TelemetryArchive.promiseArchivedPingById(pingId);
+
+  Assert.notEqual(
+    archivedCopy.payload.encryptedData,
+    payload,
+    "The encrypted payload must not match the plaintext."
+  );
+
+  Assert.equal(
+    archivedCopy.payload.pioneerId,
+    overriddenPioneerId,
+    "Pioneer ID in ping must match the provided override."
+  );
+});
+
 // Testing shutdown and checking that pings sent afterwards are rejected.
 add_task(async function test_pingRejection() {
   await TelemetryController.testReset();
@@ -853,8 +1196,11 @@ add_task(function test_scalar_filtering() {
     "test keyed scalars should be snapshotted"
   );
 
-  snapshot = Telemetry.getSnapshotForScalars("main", false, /* filter */ true)
-    .parent;
+  snapshot = Telemetry.getSnapshotForScalars(
+    "main",
+    false,
+    /* filter */ true
+  ).parent;
   keyedSnapshot = Telemetry.getSnapshotForKeyedScalars(
     "main",
     false,
