@@ -8,21 +8,20 @@
  * checked in the second request.
  */
 
-const { CommonUtils } = ChromeUtils.import(
-  "resource://services-common/utils.js"
+const { ClientID } = ChromeUtils.importESModule(
+  "resource://gre/modules/ClientID.sys.mjs"
 );
-const { ClientID } = ChromeUtils.import("resource://gre/modules/ClientID.jsm");
-const { Services } = ChromeUtils.import("resource://gre/modules/Services.jsm");
-ChromeUtils.import("resource://gre/modules/XPCOMUtils.jsm", this);
-ChromeUtils.import("resource://gre/modules/TelemetryController.jsm", this);
-ChromeUtils.import("resource://gre/modules/TelemetrySession.jsm", this);
-ChromeUtils.import("resource://gre/modules/TelemetryStorage.jsm", this);
-ChromeUtils.import("resource://gre/modules/TelemetryEnvironment.jsm", this);
-ChromeUtils.import("resource://gre/modules/TelemetrySend.jsm", this);
-ChromeUtils.import("resource://gre/modules/TelemetryUtils.jsm", this);
-ChromeUtils.import("resource://gre/modules/TelemetryReportingPolicy.jsm", this);
-const { Preferences } = ChromeUtils.import(
-  "resource://gre/modules/Preferences.jsm"
+const { TelemetrySession } = ChromeUtils.importESModule(
+  "resource://gre/modules/TelemetrySession.sys.mjs"
+);
+const { TelemetryEnvironment } = ChromeUtils.importESModule(
+  "resource://gre/modules/TelemetryEnvironment.sys.mjs"
+);
+const { TelemetryReportingPolicy } = ChromeUtils.importESModule(
+  "resource://gre/modules/TelemetryReportingPolicy.sys.mjs"
+);
+const { Preferences } = ChromeUtils.importESModule(
+  "resource://gre/modules/Preferences.sys.mjs"
 );
 
 const PING_FORMAT_VERSION = 4;
@@ -36,12 +35,6 @@ const REASON_TEST_PING = "test-ping";
 const REASON_DAILY = "daily";
 const REASON_ENVIRONMENT_CHANGE = "environment-change";
 
-const PLATFORM_VERSION = "1.9.2";
-const APP_VERSION = "1";
-const APP_ID = "xpcshell@tests.mozilla.org";
-const APP_NAME = "XPCShell";
-
-const IGNORE_HISTOGRAM_TO_CLONE = "MEMORY_HEAP_ALLOCATED";
 const IGNORE_CLONED_HISTOGRAM = "test::ignore_me_also";
 // Add some unicode characters here to ensure that sending them works correctly.
 const SHUTDOWN_TIME = 10000;
@@ -53,9 +46,6 @@ const PR_CREATE_FILE = 0x8;
 const PR_TRUNCATE = 0x20;
 const RW_OWNER = parseInt("0600", 8);
 
-const NUMBER_OF_THREADS_TO_LAUNCH = 30;
-var gNumberOfThreadsLaunched = 0;
-
 const MS_IN_ONE_HOUR = 60 * 60 * 1000;
 const MS_IN_ONE_DAY = 24 * MS_IN_ONE_HOUR;
 
@@ -63,8 +53,8 @@ const DATAREPORTING_DIR = "datareporting";
 const ABORTED_PING_FILE_NAME = "aborted-session-ping";
 const ABORTED_SESSION_UPDATE_INTERVAL_MS = 5 * 60 * 1000;
 
-XPCOMUtils.defineLazyGetter(this, "DATAREPORTING_PATH", function() {
-  return OS.Path.join(OS.Constants.Path.profileDir, DATAREPORTING_DIR);
+XPCOMUtils.defineLazyGetter(this, "DATAREPORTING_PATH", function () {
+  return PathUtils.join(PathUtils.profileDir, DATAREPORTING_DIR);
 });
 
 var gClientID = null;
@@ -81,20 +71,15 @@ function sendPing() {
 }
 
 function fakeGenerateUUID(sessionFunc, subsessionFunc) {
-  let session = ChromeUtils.import(
-    "resource://gre/modules/TelemetrySession.jsm",
-    null
+  const { Policy } = ChromeUtils.importESModule(
+    "resource://gre/modules/TelemetrySession.sys.mjs"
   );
-  session.Policy.generateSessionUUID = sessionFunc;
-  session.Policy.generateSubsessionUUID = subsessionFunc;
+  Policy.generateSessionUUID = sessionFunc;
+  Policy.generateSubsessionUUID = subsessionFunc;
 }
 
 function fakeIdleNotification(topic) {
-  let scheduler = ChromeUtils.import(
-    "resource://gre/modules/TelemetryScheduler.jsm",
-    null
-  );
-  return scheduler.TelemetryScheduler.observe(null, topic, null);
+  return TelemetryScheduler.observe(null, topic, null);
 }
 
 function setupTestData() {
@@ -108,21 +93,6 @@ function setupTestData() {
   k1.add("b");
 }
 
-function getSavedPingFile(basename) {
-  let tmpDir = Services.dirsvc.get("ProfD", Ci.nsIFile);
-  let pingFile = tmpDir.clone();
-  pingFile.append(basename);
-  if (pingFile.exists()) {
-    pingFile.remove(true);
-  }
-  registerCleanupFunction(function() {
-    try {
-      pingFile.remove(true);
-    } catch (e) {}
-  });
-  return pingFile;
-}
-
 function checkPingFormat(aPing, aType, aHasClientId, aHasEnvironment) {
   const MANDATORY_PING_FIELDS = [
     "type",
@@ -207,7 +177,8 @@ function checkPayloadInfo(data, reason) {
   let isoDateCheck = arg => {
     // We expect use of this version of the ISO format:
     // 2015-04-12T18:51:19.1+00:00
-    const isoDateRegEx = /^\d{4}-\d{2}-\d{2}T\d{2}:\d{2}:\d{2}\.\d+[+-]\d{2}:\d{2}$/;
+    const isoDateRegEx =
+      /^\d{4}-\d{2}-\d{2}T\d{2}:\d{2}:\d{2}\.\d+[+-]\d{2}:\d{2}$/;
     return (
       stringCheck(arg) &&
       !Number.isNaN(Date.parse(arg)) &&
@@ -247,7 +218,8 @@ function checkPayloadInfo(data, reason) {
 
   // Check for a valid revision.
   if (data.revision != "") {
-    const revisionUrlRegEx = /^http[s]?:\/\/hg.mozilla.org(\/[a-z\S]+)+(\/rev\/[0-9a-z]+)$/g;
+    const revisionUrlRegEx =
+      /^http[s]?:\/\/hg.mozilla.org(\/[a-z\S]+)+(\/rev\/[0-9a-z]+)$/g;
     Assert.ok(revisionUrlRegEx.test(data.revision));
   }
 
@@ -266,12 +238,17 @@ function checkPayloadInfo(data, reason) {
     Date.parse(data.subsessionStartDate) >= Date.parse(data.sessionStartDate)
   );
   Assert.ok(data.profileSubsessionCounter >= data.subsessionCounter);
+
+  // According to https://en.wikipedia.org/wiki/List_of_UTC_time_offsets,
+  // UTC offsets range from -12 to +14 hours.
+  // Don't think the extremes of the range are affected by further
+  // daylight-savings adjustments, but it is possible.
   Assert.ok(
     data.timezoneOffset >= -12 * 60,
     "The timezone must be in a valid range."
   );
   Assert.ok(
-    data.timezoneOffset <= 12 * 60,
+    data.timezoneOffset <= 14 * 60,
     "The timezone must be in a valid range."
   );
 }
@@ -298,7 +275,7 @@ function checkScalars(processes) {
     "The keyedScalars entry must be an object."
   );
 
-  let checkScalar = function(scalar) {
+  let checkScalar = function (scalar, name) {
     // Check if the value is of a supported type.
     const valueType = typeof scalar;
     switch (valueType) {
@@ -329,7 +306,7 @@ function checkScalars(processes) {
   const scalars = parentProcess.scalars;
   for (let name in scalars) {
     Assert.equal(typeof name, "string", "Scalar names must be strings.");
-    checkScalar(scalars[name]);
+    checkScalar(scalars[name], name);
   }
 
   // Check that we have valid keyed scalar entries.
@@ -346,7 +323,7 @@ function checkScalars(processes) {
         key.length <= 70,
         "Keyed scalar keys can't have more than 70 characters."
       );
-      checkScalar(scalars[name][key]);
+      checkScalar(scalars[name][key], name);
     }
   }
 }
@@ -358,11 +335,6 @@ function checkPayload(payload, reason, successfulPings) {
   Assert.ok(payload.simpleMeasurements.totalTime >= 0);
   Assert.equal(payload.simpleMeasurements.startupInterrupted, 1);
   Assert.equal(payload.simpleMeasurements.shutdownDuration, SHUTDOWN_TIME);
-  Assert.ok("maximalNumberOfConcurrentThreads" in payload.simpleMeasurements);
-  Assert.ok(
-    payload.simpleMeasurements.maximalNumberOfConcurrentThreads >=
-      gNumberOfThreadsLaunched
-  );
 
   let activeTicks = payload.simpleMeasurements.activeTicks;
   Assert.ok(activeTicks >= 0);
@@ -457,11 +429,6 @@ function checkPayload(payload, reason, successfulPings) {
     "mainThread" in payload.slowSQL && "otherThreads" in payload.slowSQL
   );
 
-  Assert.ok(
-    "IceCandidatesStats" in payload.webrtc &&
-      "webrtc" in payload.webrtc.IceCandidatesStats
-  );
-
   // Check keyed histogram payload.
 
   Assert.ok("keyedHistograms" in payload);
@@ -536,7 +503,7 @@ function write_fake_failedprofilelocks_file() {
 add_task(async function test_setup() {
   // Addon manager needs a profile directory
   do_get_profile();
-  loadAddonManager(APP_ID, APP_NAME, APP_VERSION, PLATFORM_VERSION);
+  await loadAddonManager(APP_ID, APP_NAME, APP_VERSION, PLATFORM_VERSION);
   finishAddonManagerStartup();
   fakeIntlReady();
   // Make sure we don't generate unexpected pings due to pref changes.
@@ -550,30 +517,6 @@ add_task(async function test_setup() {
   // Make it look like we've shutdown before.
   write_fake_shutdown_file();
 
-  let currentMaxNumberOfThreads = Telemetry.maximalNumberOfConcurrentThreads;
-  Assert.ok(currentMaxNumberOfThreads > 0);
-
-  // Try to augment the maximal number of threads currently launched
-  let threads = [];
-  try {
-    for (let i = 0; i < currentMaxNumberOfThreads + 10; ++i) {
-      threads.push(Services.tm.newThread(0));
-    }
-  } catch (ex) {
-    // If memory is too low, it is possible that not all threads will be launched.
-  }
-  gNumberOfThreadsLaunched = threads.length;
-
-  Assert.ok(
-    Telemetry.maximalNumberOfConcurrentThreads >= gNumberOfThreadsLaunched
-  );
-
-  registerCleanupFunction(function() {
-    threads.forEach(function(thread) {
-      thread.shutdown();
-    });
-  });
-
   await new Promise(resolve =>
     Telemetry.asyncFetchTelemetryData(wrapWithExceptionHandler(resolve))
   );
@@ -597,6 +540,64 @@ add_task(async function test_expiredHistogram() {
   );
 });
 
+add_task(async function sessionTimeExcludingAndIncludingSuspend() {
+  if (gIsAndroid) {
+    // We don't support this new probe on android at the moment.
+    return;
+  }
+  Preferences.set("toolkit.telemetry.testing.overrideProductsCheck", true);
+  await TelemetryController.testReset();
+  let subsession = TelemetrySession.getPayload("environment-change", true);
+  let parentScalars = subsession.processes.parent.scalars;
+
+  let withSuspend =
+    parentScalars["browser.engagement.session_time_including_suspend"];
+  let withoutSuspend =
+    parentScalars["browser.engagement.session_time_excluding_suspend"];
+
+  Assert.ok(
+    withSuspend > 0,
+    "The session time including suspend should be positive"
+  );
+
+  Assert.ok(
+    withoutSuspend > 0,
+    "The session time excluding suspend should be positive"
+  );
+
+  // Two things about the next assertion:
+  // 1. The two calls to get the two different uptime values are made
+  //    separately, so we can't guarantee equality, even if we know the machine
+  //    has not been suspended (for example because it's running in infra and
+  //    was just booted). In this case the value should be close to each other.
+  // 2. This test will fail if the device running this has been suspended in
+  //    between booting the Firefox process running this test, and doing the
+  //    following assertion test, but that's unlikely in practice.
+  const max_delta_ms = 100;
+
+  Assert.ok(
+    withSuspend - withoutSuspend <= max_delta_ms,
+    "In test condition, the two uptimes should be close to each other"
+  );
+
+  // This however should always hold, except on Windows < 10, where the two
+  // clocks are from different system calls, and it can fail in test condition
+  // because the machine has not been suspended.
+  if (
+    AppConstants.platform != "win" ||
+    AppConstants.isPlatformAndVersionAtLeast("win", "10.0")
+  ) {
+    Assert.greaterOrEqual(
+      withSuspend,
+      withoutSuspend,
+      `The uptime with suspend must always been greater or equal to the uptime
+       without suspend`
+    );
+  }
+
+  Preferences.set("toolkit.telemetry.testing.overrideProductsCheck", false);
+});
+
 // Sends a ping to a non existing server. If we remove this test, we won't get
 // all the histograms we need in the main ping.
 add_task(async function test_noServerPing() {
@@ -625,7 +626,10 @@ add_task(async function test_simplePing() {
 
   const expectedSessionUUID = "bd314d15-95bf-4356-b682-b6c4a8942202";
   const expectedSubsessionUUID = "3e2e5f6c-74ba-4e4d-a93f-a48af238a8c7";
-  fakeGenerateUUID(() => expectedSessionUUID, () => expectedSubsessionUUID);
+  fakeGenerateUUID(
+    () => expectedSessionUUID,
+    () => expectedSubsessionUUID
+  );
   await TelemetryController.testReset();
 
   // Session and subsession start dates are faked during TelemetrySession setup. We can
@@ -732,7 +736,7 @@ add_task(async function test_checkSubsessionScalars() {
   }
   // No scalar must be reported in classic pings (e.g. saved-session).
   Assert.ok(
-    Object.keys(classic.processes.parent.scalars).length == 0,
+    !Object.keys(classic.processes.parent.scalars).length,
     "Scalars must not be reported in a classic ping."
   );
 
@@ -794,7 +798,10 @@ add_task(async function test_dailyCollection() {
   fakeNow(now);
 
   // Fake scheduler functions to control daily collection flow in tests.
-  fakeSchedulerTimer(callback => (schedulerTickCallback = callback), () => {});
+  fakeSchedulerTimer(
+    callback => (schedulerTickCallback = callback),
+    () => {}
+  );
 
   // Init and check timer.
   await TelemetryStorage.testClearPendingPings();
@@ -896,7 +903,10 @@ add_task(async function test_dailyDuplication() {
   let now = new Date(2030, 1, 1, 0, 0, 0);
   fakeNow(now);
   // Fake scheduler functions to control daily collection flow in tests.
-  fakeSchedulerTimer(callback => (schedulerTickCallback = callback), () => {});
+  fakeSchedulerTimer(
+    callback => (schedulerTickCallback = callback),
+    () => {}
+  );
   await TelemetryController.testReset();
 
   // Make sure the daily ping gets triggered at midnight.
@@ -949,7 +959,10 @@ add_task(async function test_dailyOverdue() {
   let now = new Date(2030, 1, 1, 11, 0, 0);
   fakeNow(now);
   // Fake scheduler functions to control daily collection flow in tests.
-  fakeSchedulerTimer(callback => (schedulerTickCallback = callback), () => {});
+  fakeSchedulerTimer(
+    callback => (schedulerTickCallback = callback),
+    () => {}
+  );
   await TelemetryStorage.testClearPendingPings();
   await TelemetryController.testReset();
 
@@ -1188,8 +1201,8 @@ add_task(async function test_savedPingsOnShutdown() {
   // Assure that we store the ping properly when saving sessions on shutdown.
   // We make the TelemetryController shutdown to trigger a session save.
   const dir = TelemetryStorage.pingDirectoryPath;
-  await OS.File.removeDir(dir, { ignoreAbsent: true });
-  await OS.File.makeDir(dir);
+  await IOUtils.remove(dir, { ignoreAbsent: true, recursive: true });
+  await IOUtils.makeDirectory(dir);
   await TelemetryController.testShutdown();
 
   PingServer.clearRequests();
@@ -1208,7 +1221,7 @@ add_task(async function test_savedPingsOnShutdown() {
 add_task(async function test_sendShutdownPing() {
   if (
     gIsAndroid ||
-    (AppConstants.platform == "linux" && OS.Constants.Sys.bits == 32)
+    (AppConstants.platform == "linux" && !Services.appinfo.is64Bit)
   ) {
     // We don't support the pingsender on Android, yet, see bug 1335917.
     // We also don't suppor the pingsender testing on Treeherder for
@@ -1217,7 +1230,7 @@ add_task(async function test_sendShutdownPing() {
     return;
   }
 
-  let checkPendingShutdownPing = async function() {
+  let checkPendingShutdownPing = async function () {
     let pendingPings = await TelemetryStorage.loadPendingPingList();
     Assert.equal(pendingPings.length, 1, "We expect 1 pending ping: shutdown.");
     // Load the pings off the disk.
@@ -1347,7 +1360,7 @@ add_task(async function test_sendShutdownPing() {
 add_task(async function test_sendFirstShutdownPing() {
   if (
     gIsAndroid ||
-    (AppConstants.platform == "linux" && OS.Constants.Sys.bits == 32)
+    (AppConstants.platform == "linux" && !Services.appinfo.is64Bit)
   ) {
     // We don't support the pingsender on Android, yet, see bug 1335917.
     // We also don't suppor the pingsender testing on Treeherder for
@@ -1356,7 +1369,7 @@ add_task(async function test_sendFirstShutdownPing() {
     return;
   }
 
-  let storageContainsFirstShutdown = async function() {
+  let storageContainsFirstShutdown = async function () {
     let pendingPings = await TelemetryStorage.loadPendingPingList();
     let pings = await Promise.all(
       pendingPings.map(async p => {
@@ -1366,7 +1379,7 @@ add_task(async function test_sendFirstShutdownPing() {
     return pings.find(p => p.type == "first-shutdown");
   };
 
-  let checkShutdownNotSent = async function() {
+  let checkShutdownNotSent = async function () {
     // The failure-mode of the ping-sender is used to check that a ping was
     // *not* sent. This can be combined with the state of the storage to infer
     // the appropriate behavior from the preference flags.
@@ -1486,19 +1499,19 @@ add_task(async function test_sendFirstShutdownPing() {
 add_task(async function test_savedSessionData() {
   // Create the directory which will contain the data file, if it doesn't already
   // exist.
-  await OS.File.makeDir(DATAREPORTING_PATH);
+  await IOUtils.makeDirectory(DATAREPORTING_PATH);
   getHistogram("TELEMETRY_SESSIONDATA_FAILED_LOAD").clear();
   getHistogram("TELEMETRY_SESSIONDATA_FAILED_PARSE").clear();
   getHistogram("TELEMETRY_SESSIONDATA_FAILED_VALIDATION").clear();
 
   // Write test data to the session data file.
-  const dataFilePath = OS.Path.join(DATAREPORTING_PATH, "session-state.json");
+  const dataFilePath = PathUtils.join(DATAREPORTING_PATH, "session-state.json");
   const sessionState = {
     sessionId: null,
     subsessionId: null,
     profileSubsessionCounter: 3785,
   };
-  await CommonUtils.writeJSON(sessionState, dataFilePath);
+  await IOUtils.writeJSON(dataFilePath, sessionState);
 
   const PREF_TEST = "toolkit.telemetry.test.pref1";
   Preferences.reset(PREF_TEST);
@@ -1511,7 +1524,10 @@ add_task(async function test_savedSessionData() {
   const expectedSubsessions = sessionState.profileSubsessionCounter + 2;
   const expectedSessionUUID = "ff602e52-47a1-b7e8-4c1a-ffffffffc87a";
   const expectedSubsessionUUID = "009fd1ad-b85e-4817-b3e5-000000003785";
-  fakeGenerateUUID(() => expectedSessionUUID, () => expectedSubsessionUUID);
+  fakeGenerateUUID(
+    () => expectedSessionUUID,
+    () => expectedSubsessionUUID
+  );
 
   if (gIsAndroid) {
     // We don't support subsessions yet on Android, so skip the next checks.
@@ -1546,7 +1562,7 @@ add_task(async function test_savedSessionData() {
   fakeGenerateUUID(TelemetryUtils.generateUUID, TelemetryUtils.generateUUID);
 
   // Load back the serialised session data.
-  let data = await CommonUtils.readJSON(dataFilePath);
+  let data = await IOUtils.readJSON(dataFilePath);
   Assert.equal(data.profileSubsessionCounter, expectedSubsessions);
   Assert.equal(data.sessionId, expectedSessionUUID);
   Assert.equal(data.subsessionId, expectedSubsessionUUID);
@@ -1558,20 +1574,23 @@ add_task(async function test_sessionData_ShortSession() {
     return;
   }
 
-  const SESSION_STATE_PATH = OS.Path.join(
+  const SESSION_STATE_PATH = PathUtils.join(
     DATAREPORTING_PATH,
     "session-state.json"
   );
 
   // Remove the session state file.
-  await OS.File.remove(SESSION_STATE_PATH, { ignoreAbsent: true });
+  await IOUtils.remove(SESSION_STATE_PATH, { ignoreAbsent: true });
   getHistogram("TELEMETRY_SESSIONDATA_FAILED_LOAD").clear();
   getHistogram("TELEMETRY_SESSIONDATA_FAILED_PARSE").clear();
   getHistogram("TELEMETRY_SESSIONDATA_FAILED_VALIDATION").clear();
 
   const expectedSessionUUID = "ff602e52-47a1-b7e8-4c1a-ffffffffc87a";
   const expectedSubsessionUUID = "009fd1ad-b85e-4817-b3e5-000000003785";
-  fakeGenerateUUID(() => expectedSessionUUID, () => expectedSubsessionUUID);
+  fakeGenerateUUID(
+    () => expectedSessionUUID,
+    () => expectedSubsessionUUID
+  );
 
   // We intentionally don't wait for the setup to complete and shut down to simulate
   // short sessions. We expect the profile subsession counter to be 1.
@@ -1604,17 +1623,16 @@ add_task(async function test_sessionData_ShortSession() {
 add_task(async function test_invalidSessionData() {
   // Create the directory which will contain the data file, if it doesn't already
   // exist.
-  await OS.File.makeDir(DATAREPORTING_PATH);
+  await IOUtils.makeDirectory(DATAREPORTING_PATH);
   getHistogram("TELEMETRY_SESSIONDATA_FAILED_LOAD").clear();
   getHistogram("TELEMETRY_SESSIONDATA_FAILED_PARSE").clear();
   getHistogram("TELEMETRY_SESSIONDATA_FAILED_VALIDATION").clear();
 
   // Write test data to the session data file. This should fail to parse.
-  const dataFilePath = OS.Path.join(DATAREPORTING_PATH, "session-state.json");
+  const dataFilePath = PathUtils.join(DATAREPORTING_PATH, "session-state.json");
   const unparseableData = "{asdf:@äü";
-  OS.File.writeAtomic(dataFilePath, unparseableData, {
-    encoding: "utf-8",
-    tmpPath: dataFilePath + ".tmp",
+  await IOUtils.writeUTF8(dataFilePath, unparseableData, {
+    tmpPath: `${dataFilePath}.tmp`,
   });
 
   // Start TelemetryController so that it loads the session data file.
@@ -1630,13 +1648,16 @@ add_task(async function test_invalidSessionData() {
     profileSubsessionCounter: "not-a-number?",
     someOtherField: 12,
   };
-  await CommonUtils.writeJSON(sessionState, dataFilePath);
+  await IOUtils.writeJSON(dataFilePath, sessionState);
 
   // The session data file should not load. Only expect the current subsession.
   const expectedSubsessions = 1;
   const expectedSessionUUID = "ff602e52-47a1-b7e8-4c1a-ffffffffc87a";
   const expectedSubsessionUUID = "009fd1ad-b85e-4817-b3e5-000000003785";
-  fakeGenerateUUID(() => expectedSessionUUID, () => expectedSubsessionUUID);
+  fakeGenerateUUID(
+    () => expectedSessionUUID,
+    () => expectedSubsessionUUID
+  );
 
   // Start TelemetryController so that it loads the session data file.
   await TelemetryController.testShutdown();
@@ -1654,7 +1675,7 @@ add_task(async function test_invalidSessionData() {
   fakeGenerateUUID(TelemetryUtils.generateUUID, TelemetryUtils.generateUUID);
 
   // Load back the serialised session data.
-  let data = await CommonUtils.readJSON(dataFilePath);
+  let data = await IOUtils.readJSON(dataFilePath);
   Assert.equal(data.profileSubsessionCounter, expectedSubsessions);
   Assert.equal(data.sessionId, expectedSessionUUID);
   Assert.equal(data.subsessionId, expectedSubsessionUUID);
@@ -1666,20 +1687,29 @@ add_task(async function test_abortedSession() {
     return;
   }
 
-  const ABORTED_FILE = OS.Path.join(DATAREPORTING_PATH, ABORTED_PING_FILE_NAME);
+  const ABORTED_FILE = PathUtils.join(
+    DATAREPORTING_PATH,
+    ABORTED_PING_FILE_NAME
+  );
 
   // Make sure the aborted sessions directory does not exist to test its creation.
-  await OS.File.removeDir(DATAREPORTING_PATH, { ignoreAbsent: true });
+  await IOUtils.remove(DATAREPORTING_PATH, {
+    ignoreAbsent: true,
+    recursive: true,
+  });
 
   let schedulerTickCallback = null;
   let now = new Date(2040, 1, 1, 0, 0, 0);
   fakeNow(now);
   // Fake scheduler functions to control aborted-session flow in tests.
-  fakeSchedulerTimer(callback => (schedulerTickCallback = callback), () => {});
+  fakeSchedulerTimer(
+    callback => (schedulerTickCallback = callback),
+    () => {}
+  );
   await TelemetryController.testReset();
 
   Assert.ok(
-    await OS.File.exists(DATAREPORTING_PATH),
+    await IOUtils.exists(DATAREPORTING_PATH),
     "Telemetry must create the aborted session directory when starting."
   );
 
@@ -1692,14 +1722,13 @@ add_task(async function test_abortedSession() {
   await schedulerTickCallback();
   // Check that the aborted session is due at the correct time.
   Assert.ok(
-    await OS.File.exists(ABORTED_FILE),
+    await IOUtils.exists(ABORTED_FILE),
     "There must be an aborted session ping."
   );
 
   // This ping is not yet in the pending pings folder, so we can't access it using
   // TelemetryStorage.popPendingPings().
-  let pingContent = await OS.File.read(ABORTED_FILE, { encoding: "utf-8" });
-  let abortedSessionPing = JSON.parse(pingContent);
+  let abortedSessionPing = await IOUtils.readJSON(ABORTED_FILE);
 
   // Validate the ping.
   checkPingFormat(abortedSessionPing, PING_TYPE_MAIN, true, true);
@@ -1710,8 +1739,7 @@ add_task(async function test_abortedSession() {
   fakeNow(now);
   await schedulerTickCallback();
 
-  pingContent = await OS.File.read(ABORTED_FILE, { encoding: "utf-8" });
-  let updatedAbortedSessionPing = JSON.parse(pingContent);
+  let updatedAbortedSessionPing = await IOUtils.readJSON(ABORTED_FILE);
   checkPingFormat(updatedAbortedSessionPing, PING_TYPE_MAIN, true, true);
   Assert.equal(
     updatedAbortedSessionPing.payload.info.reason,
@@ -1725,7 +1753,7 @@ add_task(async function test_abortedSession() {
 
   await TelemetryController.testShutdown();
   Assert.ok(
-    !(await OS.File.exists(ABORTED_FILE)),
+    !(await IOUtils.exists(ABORTED_FILE)),
     "No aborted session ping must be available after a shutdown."
   );
 });
@@ -1736,16 +1764,22 @@ add_task(async function test_abortedSession_Shutdown() {
     return;
   }
 
-  const ABORTED_FILE = OS.Path.join(DATAREPORTING_PATH, ABORTED_PING_FILE_NAME);
+  const ABORTED_FILE = PathUtils.join(
+    DATAREPORTING_PATH,
+    ABORTED_PING_FILE_NAME
+  );
 
   let schedulerTickCallback = null;
   let now = fakeNow(2040, 1, 1, 0, 0, 0);
   // Fake scheduler functions to control aborted-session flow in tests.
-  fakeSchedulerTimer(callback => (schedulerTickCallback = callback), () => {});
+  fakeSchedulerTimer(
+    callback => (schedulerTickCallback = callback),
+    () => {}
+  );
   await TelemetryController.testReset();
 
   Assert.ok(
-    await OS.File.exists(DATAREPORTING_PATH),
+    await IOUtils.exists(DATAREPORTING_PATH),
     "Telemetry must create the aborted session directory when starting."
   );
 
@@ -1757,13 +1791,13 @@ add_task(async function test_abortedSession_Shutdown() {
   await schedulerTickCallback();
   // Check that the aborted session is due at the correct time.
   Assert.ok(
-    await OS.File.exists(ABORTED_FILE),
+    await IOUtils.exists(ABORTED_FILE),
     "There must be an aborted session ping."
   );
 
   // Remove the aborted session file and then shut down to make sure exceptions (e.g file
   // not found) do not compromise the shutdown.
-  await OS.File.remove(ABORTED_FILE);
+  await IOUtils.remove(ABORTED_FILE);
 
   await TelemetryController.testShutdown();
 });
@@ -1774,10 +1808,16 @@ add_task(async function test_abortedDailyCoalescing() {
     return;
   }
 
-  const ABORTED_FILE = OS.Path.join(DATAREPORTING_PATH, ABORTED_PING_FILE_NAME);
+  const ABORTED_FILE = PathUtils.join(
+    DATAREPORTING_PATH,
+    ABORTED_PING_FILE_NAME
+  );
 
   // Make sure the aborted sessions directory does not exist to test its creation.
-  await OS.File.removeDir(DATAREPORTING_PATH, { ignoreAbsent: true });
+  await IOUtils.remove(DATAREPORTING_PATH, {
+    ignoreAbsent: true,
+    recursive: true,
+  });
 
   let schedulerTickCallback = null;
   PingServer.clearRequests();
@@ -1786,13 +1826,16 @@ add_task(async function test_abortedDailyCoalescing() {
   fakeNow(nowDate);
 
   // Fake scheduler functions to control aborted-session flow in tests.
-  fakeSchedulerTimer(callback => (schedulerTickCallback = callback), () => {});
+  fakeSchedulerTimer(
+    callback => (schedulerTickCallback = callback),
+    () => {}
+  );
   await TelemetryStorage.testClearPendingPings();
   PingServer.clearRequests();
   await TelemetryController.testReset();
 
   Assert.ok(
-    await OS.File.exists(DATAREPORTING_PATH),
+    await IOUtils.exists(DATAREPORTING_PATH),
     "Telemetry must create the aborted session directory when starting."
   );
 
@@ -1811,14 +1854,13 @@ add_task(async function test_abortedDailyCoalescing() {
 
   // Check that an aborted session ping was also written to disk.
   Assert.ok(
-    await OS.File.exists(ABORTED_FILE),
+    await IOUtils.exists(ABORTED_FILE),
     "There must be an aborted session ping."
   );
 
   // Read aborted session ping and check that the session/subsession ids equal the
   // ones in the daily ping.
-  let pingContent = await OS.File.read(ABORTED_FILE, { encoding: "utf-8" });
-  let abortedSessionPing = JSON.parse(pingContent);
+  let abortedSessionPing = await IOUtils.readJSON(ABORTED_FILE);
   Assert.equal(
     abortedSessionPing.payload.info.sessionId,
     dailyPing.payload.info.sessionId
@@ -1837,7 +1879,10 @@ add_task(async function test_schedulerComputerSleep() {
     return;
   }
 
-  const ABORTED_FILE = OS.Path.join(DATAREPORTING_PATH, ABORTED_PING_FILE_NAME);
+  const ABORTED_FILE = PathUtils.join(
+    DATAREPORTING_PATH,
+    ABORTED_PING_FILE_NAME
+  );
 
   await TelemetryController.testReset();
   await TelemetryController.testShutdown();
@@ -1845,12 +1890,18 @@ add_task(async function test_schedulerComputerSleep() {
   PingServer.clearRequests();
 
   // Remove any aborted-session ping from the previous tests.
-  await OS.File.removeDir(DATAREPORTING_PATH, { ignoreAbsent: true });
+  await IOUtils.remove(DATAREPORTING_PATH, {
+    ignoreAbsent: true,
+    recursive: true,
+  });
 
   // Set a fake current date and start Telemetry.
   let nowDate = fakeNow(2009, 10, 18, 0, 0, 0);
   let schedulerTickCallback = null;
-  fakeSchedulerTimer(callback => (schedulerTickCallback = callback), () => {});
+  fakeSchedulerTimer(
+    callback => (schedulerTickCallback = callback),
+    () => {}
+  );
   await TelemetryController.testReset();
 
   // Set the current time 3 days in the future at midnight, before running the callback.
@@ -1872,7 +1923,7 @@ add_task(async function test_schedulerComputerSleep() {
   );
 
   Assert.ok(
-    await OS.File.exists(ABORTED_FILE),
+    await IOUtils.exists(ABORTED_FILE),
     "There must be an aborted session ping."
   );
 
@@ -1919,6 +1970,8 @@ add_task(async function test_schedulerEnvironmentReschedules() {
   await TelemetryController.testReset();
   await TelemetryController.testShutdown();
   await TelemetryStorage.testClearPendingPings();
+  // bug 1829855 - Sometimes the idle dispatch from a previous test interferes.
+  TelemetryScheduler.testReset();
   PingServer.clearRequests();
 
   // Set a fake current date and start Telemetry.
@@ -1927,7 +1980,10 @@ add_task(async function test_schedulerEnvironmentReschedules() {
     gMonotonicNow + 10 * MILLISECONDS_PER_MINUTE
   );
   let schedulerTickCallback = null;
-  fakeSchedulerTimer(callback => (schedulerTickCallback = callback), () => {});
+  fakeSchedulerTimer(
+    callback => (schedulerTickCallback = callback),
+    () => {}
+  );
   await TelemetryController.testReset();
   await TelemetryEnvironment.testWatchPreferences(PREFS_TO_WATCH);
 
@@ -1941,14 +1997,20 @@ add_task(async function test_schedulerEnvironmentReschedules() {
   Preferences.set(PREF_TEST, 1);
 
   // Wait for the environment-changed ping.
-  await PingServer.promiseNextPing();
+  let ping = await PingServer.promiseNextPing();
+  Assert.equal(ping.type, "main", `Expected 'main' ping on ${ping.id}`);
+  Assert.equal(
+    ping.payload.info.reason,
+    "environment-change",
+    `Expected 'environment-change' reason on ${ping.id}`
+  );
 
   // We don't expect to receive any daily ping in this test, so assert if we do.
   PingServer.registerPingHandler((req, res) => {
     const receivedPing = decodeRequestPayload(req);
     Assert.ok(
       false,
-      `No ping should be received in this test (got ${receivedPing.id}).`
+      `No ping should be received in this test (got ${receivedPing.id} type: ${receivedPing.type} reason: ${receivedPing.payload.info.reason}).`
     );
   });
 
@@ -1964,10 +2026,16 @@ add_task(async function test_schedulerNothingDue() {
     return;
   }
 
-  const ABORTED_FILE = OS.Path.join(DATAREPORTING_PATH, ABORTED_PING_FILE_NAME);
+  const ABORTED_FILE = PathUtils.join(
+    DATAREPORTING_PATH,
+    ABORTED_PING_FILE_NAME
+  );
 
   // Remove any aborted-session ping from the previous tests.
-  await OS.File.removeDir(DATAREPORTING_PATH, { ignoreAbsent: true });
+  await IOUtils.remove(DATAREPORTING_PATH, {
+    ignoreAbsent: true,
+    recursive: true,
+  });
   await TelemetryStorage.testClearPendingPings();
   await TelemetryController.testReset();
 
@@ -1985,7 +2053,10 @@ add_task(async function test_schedulerNothingDue() {
   let nowDate = new Date(2009, 10, 18, 11, 0, 0);
   fakeNow(nowDate);
   let schedulerTickCallback = null;
-  fakeSchedulerTimer(callback => (schedulerTickCallback = callback), () => {});
+  fakeSchedulerTimer(
+    callback => (schedulerTickCallback = callback),
+    () => {}
+  );
   await TelemetryController.testReset();
 
   // Delay the callback execution to a time when no ping should be due.
@@ -1999,7 +2070,7 @@ add_task(async function test_schedulerNothingDue() {
   await schedulerTickCallback();
 
   // Check that no aborted session ping was written to disk.
-  Assert.ok(!(await OS.File.exists(ABORTED_FILE)));
+  Assert.ok(!(await IOUtils.exists(ABORTED_FILE)));
 
   await TelemetryController.testShutdown();
   PingServer.resetPingHandler();
@@ -2012,13 +2083,8 @@ add_task(async function test_pingExtendedStats() {
     "fileIOReports",
     "lateWrites",
     "addonDetails",
-    "webrtc",
   ];
 
-  if (AppConstants.platform == "android") {
-    EXTENDED_PAYLOAD_FIELDS.push("UIMeasurements");
-  }
-
   // Reset telemetry and disable sending extended statistics.
   await TelemetryStorage.testClearPendingPings();
   PingServer.clearRequests();
@@ -2154,7 +2220,10 @@ add_task(async function test_DailyDueAndIdle() {
   let now = new Date(2030, 1, 1, 0, 0, 0);
   fakeNow(now);
   // Fake scheduler functions to control daily collection flow in tests.
-  fakeSchedulerTimer(callback => (schedulerTickCallback = callback), () => {});
+  fakeSchedulerTimer(
+    callback => (schedulerTickCallback = callback),
+    () => {}
+  );
   await TelemetryController.testReset();
 
   // Trigger the daily ping.
@@ -2200,7 +2269,10 @@ add_task(async function test_userIdleAndSchedlerTick() {
   let now = new Date(2030, 1, 1, 0, 0, 0);
   fakeNow(now);
   // Fake scheduler functions to control daily collection flow in tests.
-  fakeSchedulerTimer(callback => (schedulerTickCallback = callback), () => {});
+  fakeSchedulerTimer(
+    callback => (schedulerTickCallback = callback),
+    () => {}
+  );
   await TelemetryStorage.testClearPendingPings();
   await TelemetryController.testReset();
   PingServer.clearRequests();
@@ -2242,12 +2314,6 @@ add_task(async function test_changeThrottling() {
     return TelemetrySession.getPayload().info.subsessionCounter;
   };
 
-  const PREF_TEST = "toolkit.telemetry.test.pref1";
-  const PREFS_TO_WATCH = new Map([
-    [PREF_TEST, { what: TelemetryEnvironment.RECORD_PREF_STATE }],
-  ]);
-  Preferences.reset(PREF_TEST);
-
   let now = fakeNow(2050, 1, 2, 0, 0, 0);
   gMonotonicNow = fakeMonotonicNow(
     gMonotonicNow + 10 * MILLISECONDS_PER_MINUTE
@@ -2255,11 +2321,8 @@ add_task(async function test_changeThrottling() {
   await TelemetryController.testReset();
   Assert.equal(getSubsessionCount(), 1);
 
-  // Set the Environment preferences to watch.
-  await TelemetryEnvironment.testWatchPreferences(PREFS_TO_WATCH);
-
   // The first pref change should not trigger a notification.
-  Preferences.set(PREF_TEST, 1);
+  TelemetrySession.testOnEnvironmentChange("test", {});
   Assert.equal(getSubsessionCount(), 1);
 
   // We should get a change notification after the 5min throttling interval.
@@ -2267,13 +2330,13 @@ add_task(async function test_changeThrottling() {
   gMonotonicNow = fakeMonotonicNow(
     gMonotonicNow + 5 * MILLISECONDS_PER_MINUTE + 1
   );
-  Preferences.set(PREF_TEST, 2);
+  TelemetrySession.testOnEnvironmentChange("test", {});
   Assert.equal(getSubsessionCount(), 2);
 
   // After that, changes should be throttled again.
   now = fakeNow(futureDate(now, 1 * MILLISECONDS_PER_MINUTE));
   gMonotonicNow = fakeMonotonicNow(gMonotonicNow + 1 * MILLISECONDS_PER_MINUTE);
-  Preferences.set(PREF_TEST, 3);
+  TelemetrySession.testOnEnvironmentChange("test", {});
   Assert.equal(getSubsessionCount(), 2);
 
   // ... for 5min.
@@ -2281,13 +2344,14 @@ add_task(async function test_changeThrottling() {
   gMonotonicNow = fakeMonotonicNow(
     gMonotonicNow + 4 * MILLISECONDS_PER_MINUTE + 1
   );
-  Preferences.set(PREF_TEST, 4);
+  TelemetrySession.testOnEnvironmentChange("test", {});
   Assert.equal(getSubsessionCount(), 3);
-
-  // Unregister the listener.
-  TelemetryEnvironment.unregisterChangeListener("testWatchPrefs_throttling");
 });
 
 add_task(async function stopServer() {
+  // It is important to shut down the TelemetryController first as, due to test
+  // environment changes, failure to upload pings here during shutdown results
+  // in an infinite loop of send failures and retries.
+  await TelemetryController.testShutdown();
   await PingServer.stop();
 });
