@@ -2,23 +2,24 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this file,
  * You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-var { AppConstants } = ChromeUtils.import(
-  "resource://gre/modules/AppConstants.jsm"
+var { AppConstants } = ChromeUtils.importESModule(
+  "resource://gre/modules/AppConstants.sys.mjs"
 );
-var { Services } = ChromeUtils.import("resource://gre/modules/Services.jsm");
-var { XPCOMUtils } = ChromeUtils.import(
-  "resource://gre/modules/XPCOMUtils.jsm"
+var { XPCOMUtils } = ChromeUtils.importESModule(
+  "resource://gre/modules/XPCOMUtils.sys.mjs"
 );
 
+ChromeUtils.defineESModuleGetters(this, {
+  BrowserUtils: "resource://gre/modules/BrowserUtils.sys.mjs",
+  Deprecated: "resource://gre/modules/Deprecated.sys.mjs",
+  DownloadLastDir: "resource://gre/modules/DownloadLastDir.sys.mjs",
+  DownloadPaths: "resource://gre/modules/DownloadPaths.sys.mjs",
+  Downloads: "resource://gre/modules/Downloads.sys.mjs",
+  FileUtils: "resource://gre/modules/FileUtils.sys.mjs",
+  PrivateBrowsingUtils: "resource://gre/modules/PrivateBrowsingUtils.sys.mjs",
+});
+
 XPCOMUtils.defineLazyModuleGetters(this, {
-  BrowserUtils: "resource://gre/modules/BrowserUtils.jsm",
-  Downloads: "resource://gre/modules/Downloads.jsm",
-  DownloadPaths: "resource://gre/modules/DownloadPaths.jsm",
-  DownloadLastDir: "resource://gre/modules/DownloadLastDir.jsm",
-  FileUtils: "resource://gre/modules/FileUtils.jsm",
-  OS: "resource://gre/modules/osfile.jsm",
-  PrivateBrowsingUtils: "resource://gre/modules/PrivateBrowsingUtils.jsm",
-  Deprecated: "resource://gre/modules/Deprecated.jsm",
   NetUtil: "resource://gre/modules/NetUtil.jsm",
 });
 
@@ -31,17 +32,23 @@ var ContentAreaUtils = {
   },
 };
 
-function urlSecurityCheck(aURL, aPrincipal, aFlags) {
-  return BrowserUtils.urlSecurityCheck(aURL, aPrincipal, aFlags);
-}
-
-function forbidCPOW(arg, func, argname) {
-  if (
-    arg &&
-    (typeof arg == "object" || typeof arg == "function") &&
-    Cu.isCrossProcessWrapper(arg)
-  ) {
-    throw new Error(`no CPOWs allowed for argument ${argname} to ${func}`);
+function urlSecurityCheck(
+  aURL,
+  aPrincipal,
+  aFlags = Services.scriptSecurityManager
+) {
+  if (aURL instanceof Ci.nsIURI) {
+    Services.scriptSecurityManager.checkLoadURIWithPrincipal(
+      aPrincipal,
+      aURL,
+      aFlags
+    );
+  } else {
+    Services.scriptSecurityManager.checkLoadURIStrWithPrincipal(
+      aPrincipal,
+      aURL,
+      aFlags
+    );
   }
 }
 
@@ -65,21 +72,20 @@ function forbidCPOW(arg, func, argname) {
 //
 function saveURL(
   aURL,
+  aOriginalURL,
   aFileName,
   aFilePickerTitleKey,
   aShouldBypassCache,
   aSkipPrompt,
-  aReferrer,
+  aReferrerInfo,
+  aCookieJarSettings,
   aSourceDocument,
   aIsContentWindowPrivate,
   aPrincipal
 ) {
-  forbidCPOW(aURL, "saveURL", "aURL");
-  forbidCPOW(aReferrer, "saveURL", "aReferrer");
-  // Allow aSourceDocument to be a CPOW.
-
   internalSave(
     aURL,
+    aOriginalURL,
     null,
     aFileName,
     null,
@@ -87,7 +93,8 @@ function saveURL(
     aShouldBypassCache,
     aFilePickerTitleKey,
     null,
-    aReferrer,
+    aReferrerInfo,
+    aCookieJarSettings,
     aSourceDocument,
     aSkipPrompt,
     null,
@@ -96,169 +103,42 @@ function saveURL(
   );
 }
 
-// Just like saveURL, but will get some info off the image before
-// calling internalSave
-// Clientele: (Make sure you don't break any of these)
-//  - Context ->  Save Image As...
-const imgICache = Ci.imgICache;
-const nsISupportsCString = Ci.nsISupportsCString;
-
-/**
- * Offers to save an image URL to the file system.
- *
- * @param aURL (string)
- *        The URL of the image to be saved.
- * @param aFileName (string)
- *        The suggested filename for the saved file.
- * @param aFilePickerTitleKey (string, optional)
- *        Localized string key for an alternate title for the file
- *        picker. If set to null, this will default to something sensible.
- * @param aShouldBypassCache (bool)
- *        If true, the image will always be retrieved from the server instead
- *        of the network or image caches.
- * @param aSkipPrompt (bool)
- *        If true, we will attempt to save the file with the suggested
- *        filename to the default downloads folder without showing the
- *        file picker.
- * @param aReferrer (nsIURI, optional)
- *        The referrer URI object (not a URL string) to use, or null
- *        if no referrer should be sent.
- * @param aDoc (Document, deprecated, optional)
- *        The content document that the save is being initiated from. If this
- *        is omitted, then aIsContentWindowPrivate must be provided.
- * @param aContentType (string, optional)
- *        The content type of the image.
- * @param aContentDisp (string, optional)
- *        The content disposition of the image.
- * @param aIsContentWindowPrivate (bool)
- *        Whether or not the containing window is in private browsing mode.
- *        Does not need to be provided is aDoc is passed.
- */
-function saveImageURL(
-  aURL,
-  aFileName,
-  aFilePickerTitleKey,
-  aShouldBypassCache,
-  aSkipPrompt,
-  aReferrer,
-  aDoc,
-  aContentType,
-  aContentDisp,
-  aIsContentWindowPrivate,
-  aPrincipal
-) {
-  forbidCPOW(aURL, "saveImageURL", "aURL");
-  forbidCPOW(aReferrer, "saveImageURL", "aReferrer");
-
-  if (aDoc && aIsContentWindowPrivate == undefined) {
-    if (Cu.isCrossProcessWrapper(aDoc)) {
-      Deprecated.warning(
-        "saveImageURL should not be passed document CPOWs. " +
-          "The caller should pass in the content type and " +
-          "disposition themselves",
-        "https://bugzilla.mozilla.org/show_bug.cgi?id=1243643"
-      );
-    }
-    // This will definitely not work for in-browser code or multi-process compatible
-    // add-ons due to bug 1233497, which makes unsafe CPOW usage throw by default.
-    Deprecated.warning(
-      "saveImageURL should be passed the private state of " +
-        "the containing window.",
-      "https://bugzilla.mozilla.org/show_bug.cgi?id=1243643"
-    );
-    aIsContentWindowPrivate = PrivateBrowsingUtils.isContentWindowPrivate(
-      aDoc.defaultView
-    );
-  }
-
-  // We'd better have the private state by now.
-  if (aIsContentWindowPrivate == undefined) {
-    throw new Error(
-      "saveImageURL couldn't compute private state of content window"
-    );
-  }
-
-  if (
-    !aShouldBypassCache &&
-    (aDoc && !Cu.isCrossProcessWrapper(aDoc)) &&
-    (!aContentType && !aContentDisp)
-  ) {
-    try {
-      var imageCache = Cc["@mozilla.org/image/tools;1"]
-        .getService(Ci.imgITools)
-        .getImgCacheForDocument(aDoc);
-      var props = imageCache.findEntryProperties(
-        makeURI(aURL, getCharsetforSave(null)),
-        aDoc
-      );
-      if (props) {
-        aContentType = props.get("type", nsISupportsCString);
-        aContentDisp = props.get("content-disposition", nsISupportsCString);
-      }
-    } catch (e) {
-      // Failure to get type and content-disposition off the image is non-fatal
-    }
-  }
-
-  internalSave(
-    aURL,
-    null,
-    aFileName,
-    aContentDisp,
-    aContentType,
-    aShouldBypassCache,
-    aFilePickerTitleKey,
-    null,
-    aReferrer,
-    aDoc,
-    aSkipPrompt,
-    null,
-    aIsContentWindowPrivate,
-    aPrincipal
-  );
-}
-
-// This is like saveDocument, but takes any browser/frame-like element
-// and saves the current document inside it,
+// Save the current document inside any browser/frame-like element,
 // whether in-process or out-of-process.
-function saveBrowser(aBrowser, aSkipPrompt, aOuterWindowID = 0) {
+function saveBrowser(aBrowser, aSkipPrompt, aBrowsingContext = null) {
   if (!aBrowser) {
     throw new Error("Must have a browser when calling saveBrowser");
   }
   let persistable = aBrowser.frameLoader;
-  // Because of how pdf.js deals with principals, saving the document the "normal"
-  // way won't work. Work around this by saving the pdf's URL directly:
-  if (
-    aBrowser.contentPrincipal.URI &&
-    aBrowser.contentPrincipal.URI.spec == "resource://pdf.js/web/viewer.html" &&
-    aBrowser.currentURI.schemeIs("file")
-  ) {
-    let correctPrincipal = Services.scriptSecurityManager.createCodebasePrincipal(
-      aBrowser.currentURI,
-      aBrowser.contentPrincipal.originAttributes
-    );
-    internalSave(
-      aBrowser.currentURI.spec,
-      null /* no document */,
-      null /* automatically determine filename */,
-      null /* no content disposition */,
-      "application/pdf",
-      false /* don't bypass cache */,
-      null /* no alternative title */,
-      null /* no auto-chosen file info */,
-      null /* null referrer will be OK for file: */,
-      null /* no document */,
-      aSkipPrompt /* caller decides about prompting */,
-      null /* no cache key because the one for the document will be for pdfjs */,
-      PrivateBrowsingUtils.isWindowPrivate(aBrowser.ownerGlobal),
-      correctPrincipal
-    );
+  // PDF.js has its own way to handle saving PDFs since it may need to
+  // generate a new PDF to save modified form data.
+  if (aBrowser.contentPrincipal.spec == "resource://pdf.js/web/viewer.html") {
+    aBrowser.sendMessageToActor("PDFJS:Save", {}, "Pdfjs");
     return;
   }
   let stack = Components.stack.caller;
-  persistable.startPersistence(aOuterWindowID, {
+  persistable.startPersistence(aBrowsingContext, {
     onDocumentReady(document) {
-      saveDocument(document, aSkipPrompt);
+      if (!document || !(document instanceof Ci.nsIWebBrowserPersistDocument)) {
+        throw new Error("Must have an nsIWebBrowserPersistDocument!");
+      }
+
+      internalSave(
+        document.documentURI,
+        null, // originalURL
+        document,
+        null, // file name
+        document.contentDisposition,
+        document.contentType,
+        false, // bypass cache
+        null, // file picker title key
+        null, // chosen file data
+        document.referrerInfo,
+        document.cookieJarSettings,
+        document,
+        aSkipPrompt,
+        document.cacheKey
+      );
     },
     onError(status) {
       throw new Components.Exception(
@@ -270,68 +150,9 @@ function saveBrowser(aBrowser, aSkipPrompt, aOuterWindowID = 0) {
   });
 }
 
-// Saves a document; aDocument can be an nsIWebBrowserPersistDocument
-// (see saveBrowser, above) or a Document.
-//
-// aDocument can also be a CPOW for a remote Document, in which
-// case "save as" modes that serialize the document's DOM are
-// unavailable.  This is a temporary measure for the "Save Frame As"
-// command (bug 1141337) and pre-e10s add-ons.
-function saveDocument(aDocument, aSkipPrompt) {
-  if (!aDocument) {
-    throw new Error("Must have a document when calling saveDocument");
-  }
-
-  let contentDisposition = null;
-  let cacheKey = 0;
-
-  if (aDocument instanceof Ci.nsIWebBrowserPersistDocument) {
-    // nsIWebBrowserPersistDocument exposes these directly.
-    contentDisposition = aDocument.contentDisposition;
-    cacheKey = aDocument.cacheKey;
-  } else if (aDocument.nodeType == 9 /* DOCUMENT_NODE */) {
-    // Otherwise it's an actual document (and possibly a CPOW).
-    // We want to use cached data because the document is currently visible.
-    let win = aDocument.defaultView;
-
-    try {
-      contentDisposition = win.windowUtils.getDocumentMetadata(
-        "content-disposition"
-      );
-    } catch (ex) {
-      // Failure to get a content-disposition is ok
-    }
-
-    try {
-      let shEntry = win.docShell
-        .QueryInterface(Ci.nsIWebPageDescriptor)
-        .currentDescriptor.QueryInterface(Ci.nsISHEntry);
-
-      cacheKey = shEntry.cacheKey;
-    } catch (ex) {
-      // We might not find it in the cache.  Oh, well.
-    }
-  }
-
-  internalSave(
-    aDocument.documentURI,
-    aDocument,
-    null,
-    contentDisposition,
-    aDocument.contentType,
-    false,
-    null,
-    null,
-    aDocument.referrer ? makeURI(aDocument.referrer) : null,
-    aDocument,
-    aSkipPrompt,
-    cacheKey
-  );
-}
-
 function DownloadListener(win, transfer) {
   function makeClosure(name) {
-    return function() {
+    return function () {
       transfer[name].apply(transfer, arguments);
     };
   }
@@ -361,7 +182,7 @@ DownloadListener.prototype = {
       return ww.getPrompt(this.window, aIID);
     }
 
-    throw Cr.NS_ERROR_NO_INTERFACE;
+    throw Components.Exception("", Cr.NS_ERROR_NO_INTERFACE);
   },
 };
 
@@ -378,9 +199,8 @@ XPCOMUtils.defineConstant(this, "kSaveAsType_Text", kSaveAsType_Text);
  *  - Determines a local target filename to use
  *  - Prompts the user to confirm the destination filename and save mode
  *    (aContentType affects this)
- *  - [Note] This process involves the parameters aURL, aReferrer (to determine
- *    how aURL was encoded), aDocument, aDefaultFileName, aFilePickerTitleKey,
- *    and aSkipPrompt.
+ *  - [Note] This process involves the parameters aURL, aReferrerInfo,
+ *    aDocument, aDefaultFileName, aFilePickerTitleKey, and aSkipPrompt.
  *
  * If aChosenData is non-null, this method:
  *  - Uses the provided source URI and save file name
@@ -393,11 +213,14 @@ XPCOMUtils.defineConstant(this, "kSaveAsType_Text", kSaveAsType_Text);
  *  - Creates a 'Persist' object (which will perform the saving in the
  *    background) and then starts it.
  *  - [Note] This part of the process only involves the parameters aDocument,
- *    aShouldBypassCache and aReferrer. The source, the save name and the save
- *    mode are the ones determined previously.
+ *    aShouldBypassCache and aReferrerInfo. The source, the save name and the
+ *    save mode are the ones determined previously.
  *
  * @param aURL
  *        The String representation of the URL of the document being saved
+ * @param aOriginalURL
+ *        The String representation of the original URL of the document being
+ *        saved. It can useful in case aURL is a blob.
  * @param aDocument
  *        The document to be saved
  * @param aDefaultFileName
@@ -415,9 +238,11 @@ XPCOMUtils.defineConstant(this, "kSaveAsType_Text", kSaveAsType_Text);
  *        If non-null this contains an instance of object AutoChosen (see below)
  *        which holds pre-determined data so that the user does not need to be
  *        prompted for a target filename.
- * @param aReferrer
- *        the referrer URI object (not URL string) to use, or null
- *        if no referrer should be sent.
+ * @param aReferrerInfo
+ *        the referrerInfo object to use, or null if no referrer should be sent.
+ * @param aCookieJarSettings
+ *        the cookieJarSettings object to use. This will be used for the channel
+ *        used to save.
  * @param aInitiatingDocument [optional]
  *        The document from which the save was initiated.
  *        If this is omitted then aIsContentWindowPrivate has to be provided.
@@ -438,6 +263,7 @@ XPCOMUtils.defineConstant(this, "kSaveAsType_Text", kSaveAsType_Text);
  */
 function internalSave(
   aURL,
+  aOriginalURL,
   aDocument,
   aDefaultFileName,
   aContentDisposition,
@@ -445,18 +271,14 @@ function internalSave(
   aShouldBypassCache,
   aFilePickerTitleKey,
   aChosenData,
-  aReferrer,
+  aReferrerInfo,
+  aCookieJarSettings,
   aInitiatingDocument,
   aSkipPrompt,
   aCacheKey,
   aIsContentWindowPrivate,
   aPrincipal
 ) {
-  forbidCPOW(aURL, "internalSave", "aURL");
-  forbidCPOW(aReferrer, "internalSave", "aReferrer");
-  forbidCPOW(aCacheKey, "internalSave", "aCacheKey");
-  // Allow aInitiatingDocument to be a CPOW.
-
   if (aSkipPrompt == undefined) {
     aSkipPrompt = false;
   }
@@ -469,6 +291,7 @@ function internalSave(
   var saveMode = GetSaveModeForContentType(aContentType, aDocument);
 
   var file, sourceURI, saveAsType;
+  let contentPolicyType = Ci.nsIContentPolicy.TYPE_SAVEAS_DOWNLOAD;
   // Find the URI object for aURL and the FileName/Extension to use when saving.
   // FileName/Extension will be ignored if aChosenData supplied.
   if (aChosenData) {
@@ -493,6 +316,9 @@ function internalSave(
     );
     sourceURI = fileInfo.uri;
 
+    if (aContentType && aContentType.startsWith("image/")) {
+      contentPolicyType = Ci.nsIContentPolicy.TYPE_IMAGE;
+    }
     var fpParams = {
       fpTitleKey: aFilePickerTitleKey,
       fileInfo,
@@ -503,7 +329,7 @@ function internalSave(
     };
 
     // Find a URI to use for determining last-downloaded-to directory
-    let relatedURI = aReferrer || sourceURI;
+    let relatedURI = aReferrerInfo?.originalReferrer || sourceURI;
 
     promiseTargetFile(fpParams, aSkipPrompt, relatedURI)
       .then(aDialogAccepted => {
@@ -516,7 +342,7 @@ function internalSave(
 
         continueSave();
       })
-      .catch(Cu.reportError);
+      .catch(console.error);
   }
 
   function continueSave() {
@@ -531,7 +357,6 @@ function internalSave(
     // If we're saving a document, and are saving either in complete mode or
     // as converted text, pass the document to the web browser persist component.
     // If we're just saving the HTML (second option in the list), send only the URI.
-    let nonCPOWDocument = aDocument && !Cu.isCrossProcessWrapper(aDocument);
 
     let isPrivate = aIsContentWindowPrivate;
     if (isPrivate === undefined) {
@@ -551,16 +376,21 @@ function internalSave(
       (aDocument && (aDocument.nodePrincipal || aDocument.principal)) ||
       (aInitiatingDocument && aInitiatingDocument.nodePrincipal);
 
+    let sourceOriginalURI = aOriginalURL ? makeURI(aOriginalURL) : null;
+
     var persistArgs = {
       sourceURI,
+      sourceOriginalURI,
       sourcePrincipal,
-      sourceReferrer: aReferrer,
+      sourceReferrerInfo: aReferrerInfo,
       sourceDocument: useSaveDocument ? aDocument : null,
       targetContentType: saveAsType == kSaveAsType_Text ? "text/plain" : null,
       targetFile: file,
       sourceCacheKey: aCacheKey,
-      sourcePostData: nonCPOWDocument ? getPostData(aDocument) : null,
+      sourcePostData: aDocument ? getPostData(aDocument) : null,
       bypassCache: aShouldBypassCache,
+      contentPolicyType,
+      cookieJarSettings: aCookieJarSettings,
       isPrivate,
     };
 
@@ -576,19 +406,26 @@ function internalSave(
  * @param persistArgs.sourceURI
  *        The nsIURI of the document being saved
  * @param persistArgs.sourceCacheKey [optional]
- *        If set will be passed to savePrivacyAwareURI
+ *        If set will be passed to saveURI
  * @param persistArgs.sourceDocument [optional]
  *        The document to be saved, or null if not saving a complete document
- * @param persistArgs.sourceReferrer
+ * @param persistArgs.sourceReferrerInfo
  *        Required and used only when persistArgs.sourceDocument is NOT present,
- *        the nsIURI of the referrer to use, or null if no referrer should be
- *        sent.
+ *        the nsIReferrerInfo of the referrer info to use, or null if no
+ *        referrer should be sent.
  * @param persistArgs.sourcePostData
  *        Required and used only when persistArgs.sourceDocument is NOT present,
  *        represents the POST data to be sent along with the HTTP request, and
  *        must be null if no POST data should be sent.
  * @param persistArgs.targetFile
  *        The nsIFile of the file to create
+ * @param persistArgs.contentPolicyType
+ *        The type of content we're saving. Will be used to determine what
+ *        content is accepted, enforce sniffing restrictions, etc.
+ * @param persistArgs.cookieJarSettings [optional]
+ *        The nsICookieJarSettings that will be used for the saving channel, or
+ *        null that saveURI will create one based on the current
+ *        state of the prefs/permissions
  * @param persistArgs.targetContentType
  *        Required and used only when persistArgs.sourceDocument is present,
  *        determines the final content type of the saved file, or null to use
@@ -621,13 +458,16 @@ function internalPersist(persistArgs) {
   var tr = Cc["@mozilla.org/transfer;1"].createInstance(Ci.nsITransfer);
   tr.init(
     persistArgs.sourceURI,
+    persistArgs.sourceOriginalURI,
     targetFileURL,
     "",
     null,
     null,
     null,
     persist,
-    persistArgs.isPrivate
+    persistArgs.isPrivate,
+    Ci.nsITransfer.DOWNLOAD_ACCEPTABLE,
+    persistArgs.sourceReferrerInfo
   );
   persist.progressListener = new DownloadListener(window, tr);
 
@@ -639,11 +479,10 @@ function internalPersist(persistArgs) {
       filesFolder = persistArgs.targetFile.clone();
 
       var nameWithoutExtension = getFileBaseName(filesFolder.leafName);
-      var filesFolderLeafName = ContentAreaUtils.stringBundle.formatStringFromName(
-        "filesFolder",
-        [nameWithoutExtension],
-        1
-      );
+      var filesFolderLeafName =
+        ContentAreaUtils.stringBundle.formatStringFromName("filesFolder", [
+          nameWithoutExtension,
+        ]);
 
       filesFolder.leafName = filesFolderLeafName;
     }
@@ -667,15 +506,16 @@ function internalPersist(persistArgs) {
       kWrapColumn
     );
   } else {
-    persist.savePrivacyAwareURI(
+    persist.saveURI(
       persistArgs.sourceURI,
       persistArgs.sourcePrincipal,
       persistArgs.sourceCacheKey,
-      persistArgs.sourceReferrer,
-      Ci.nsIHttpChannel.REFERRER_POLICY_UNSET,
+      persistArgs.sourceReferrerInfo,
+      persistArgs.cookieJarSettings,
       persistArgs.sourcePostData,
       null,
       targetFileURL,
+      persistArgs.contentPolicyType || Ci.nsIContentPolicy.TYPE_SAVEAS_DOWNLOAD,
       persistArgs.isPrivate
     );
   }
@@ -740,28 +580,36 @@ function initFileInfo(
   aContentDisposition
 ) {
   try {
+    let uriExt = null;
     // Get an nsIURI object from aURL if possible:
     try {
       aFI.uri = makeURI(aURL, aURLCharset);
       // Assuming nsiUri is valid, calling QueryInterface(...) on it will
       // populate extra object fields (eg filename and file extension).
-      var url = aFI.uri.QueryInterface(Ci.nsIURL);
-      aFI.fileExt = url.fileExtension;
+      uriExt = aFI.uri.QueryInterface(Ci.nsIURL).fileExtension;
     } catch (e) {}
 
     // Get the default filename:
-    aFI.fileName = getDefaultFileName(
+    let fileName = getDefaultFileName(
       aFI.suggestedFileName || aFI.fileName,
       aFI.uri,
       aDocument,
       aContentDisposition
     );
-    // If aFI.fileExt is still blank, consider: aFI.suggestedFileName is supplied
-    // if saveURL(...) was the original caller (hence both aContentType and
+
+    let mimeService = this.getMIMEService();
+    aFI.fileName = mimeService.validateFileNameForSaving(
+      fileName,
+      aContentType,
+      mimeService.VALIDATE_FORCE_APPEND_EXTENSION
+    );
+
+    // If uriExt is blank, consider: aFI.suggestedFileName is supplied if
+    // saveURL(...) was the original caller (hence both aContentType and
     // aDocument are blank). If they were saving a link to a website then make
     // the extension .htm .
     if (
-      !aFI.fileExt &&
+      !uriExt &&
       !aDocument &&
       !aContentType &&
       /^http(s?):\/\//i.test(aURL)
@@ -769,8 +617,10 @@ function initFileInfo(
       aFI.fileExt = "htm";
       aFI.fileBaseName = aFI.fileName;
     } else {
-      aFI.fileExt = getDefaultExtension(aFI.fileName, aFI.uri, aContentType);
-      aFI.fileBaseName = getFileBaseName(aFI.fileName);
+      let idx = aFI.fileName.lastIndexOf(".");
+      aFI.fileBaseName =
+        idx >= 0 ? aFI.fileName.substring(0, idx) : aFI.fileName;
+      aFI.fileExt = idx >= 0 ? aFI.fileName.substring(idx + 1) : null;
     }
   } catch (e) {}
 }
@@ -801,7 +651,7 @@ function promiseTargetFile(
   /* optional */ aSkipPrompt,
   /* optional */ aRelatedURI
 ) {
-  return (async function() {
+  return (async function () {
     let downloadLastDir = new DownloadLastDir(window);
     let prefBranch = Services.prefs.getBranch("browser.download.");
     let useDownloadDir = prefBranch.getBoolPref("useDownloadDir");
@@ -813,34 +663,22 @@ function promiseTargetFile(
     // Default to the user's default downloads directory configured
     // through download prefs.
     let dirPath = await Downloads.getPreferredDownloadsDirectory();
-    let dirExists = await OS.File.exists(dirPath);
+    let dirExists = await IOUtils.exists(dirPath);
     let dir = new FileUtils.File(dirPath);
 
     if (useDownloadDir && dirExists) {
-      dir.append(
-        getNormalizedLeafName(aFpP.fileInfo.fileName, aFpP.fileInfo.fileExt)
-      );
+      dir.append(aFpP.fileInfo.fileName);
       aFpP.file = uniqueFile(dir);
       return true;
     }
 
     // We must prompt for the file name explicitly.
     // If we must prompt because we were asked to...
-    let file = await new Promise(resolve => {
-      if (useDownloadDir) {
-        // Keep async behavior in both branches
-        Services.tm.dispatchToMainThread(function() {
-          resolve(null);
-        });
-      } else {
-        downloadLastDir.getFileAsync(aRelatedURI, function getFileAsyncCB(
-          aFile
-        ) {
-          resolve(aFile);
-        });
-      }
-    });
-    if (file && (await OS.File.exists(file.path))) {
+    let file = null;
+    if (!useDownloadDir) {
+      file = await downloadLastDir.getFileAsync(aRelatedURI);
+    }
+    if (file && (await IOUtils.exists(file.path))) {
       dir = file;
       dirExists = true;
     }
@@ -860,10 +698,7 @@ function promiseTargetFile(
 
     fp.displayDirectory = dir;
     fp.defaultExtension = aFpP.fileInfo.fileExt;
-    fp.defaultString = getNormalizedLeafName(
-      aFpP.fileInfo.fileName,
-      aFpP.fileInfo.fileExt
-    );
+    fp.defaultString = aFpP.fileInfo.fileName;
     appendFiltersForContentType(
       fp,
       aFpP.contentType,
@@ -881,7 +716,7 @@ function promiseTargetFile(
     }
 
     let result = await new Promise(resolve => {
-      fp.open(function(aResult) {
+      fp.open(function (aResult) {
         resolve(aResult);
       });
     });
@@ -967,7 +802,7 @@ function DownloadURL(aURL, aFileName, aInitiatingDocument) {
     saveMode: SAVEMODE_FILEONLY,
   };
 
-  (async function() {
+  (async function () {
     let accepted = await promiseTargetFile(
       filepickerParams,
       true,
@@ -990,7 +825,7 @@ function DownloadURL(aURL, aFileName, aInitiatingDocument) {
     // Add the download to the list, allowing it to be managed.
     let list = await Downloads.getList(Downloads.ALL);
     list.add(download);
-  })().catch(Cu.reportError);
+  })().catch(console.error);
 }
 
 // We have no DOM, and can only save the URL as is.
@@ -1193,14 +1028,16 @@ function getDefaultFileName(
       } catch (e) {}
     }
     if (fileName) {
-      return validateFileName(fileName);
+      return Services.textToSubURI.unEscapeURIForUI(
+        fileName,
+        /* dontEscape = */ true
+      );
     }
   }
 
   let docTitle;
   if (aDocument && aDocument.title && aDocument.title.trim()) {
     // If the document looks like HTML or XML, try to use its original title.
-    docTitle = validateFileName(aDocument.title);
     let contentType = aDocument.contentType;
     if (
       contentType == "application/xhtml+xml" ||
@@ -1210,7 +1047,7 @@ function getDefaultFileName(
       contentType == "text/xml"
     ) {
       // 2) Use the document title
-      return docTitle;
+      return aDocument.title;
     }
   }
 
@@ -1218,59 +1055,52 @@ function getDefaultFileName(
     var url = aURI.QueryInterface(Ci.nsIURL);
     if (url.fileName != "") {
       // 3) Use the actual file name, if present
-      return validateFileName(
-        Services.textToSubURI.unEscapeURIForUI("UTF-8", url.fileName)
+      return Services.textToSubURI.unEscapeURIForUI(
+        url.fileName,
+        /* dontEscape = */ true
       );
     }
   } catch (e) {
     // This is something like a data: and so forth URI... no filename here.
   }
 
-  if (docTitle) {
+  // Don't use the title if it's from a data URI
+  if (docTitle && aURI?.scheme != "data") {
     // 4) Use the document title
     return docTitle;
   }
 
   if (aDefaultFileName) {
     // 5) Use the caller-provided name, if any
-    return validateFileName(aDefaultFileName);
-  }
-
-  // 6) If this is a directory, use the last directory name
-  var path = aURI.pathQueryRef.match(/\/([^\/]+)\/$/);
-  if (path && path.length > 1) {
-    return validateFileName(path[1]);
+    return aDefaultFileName;
   }
 
   try {
     if (aURI.host) {
-      // 7) Use the host.
-      return validateFileName(aURI.host);
+      // 6) Use the host.
+      return aURI.host;
     }
   } catch (e) {
     // Some files have no information at all, like Javascript generated pages
   }
-  try {
-    // 8) Use the default file name
-    return ContentAreaUtils.stringBundle.GetStringFromName(
-      "DefaultSaveFileName"
-    );
-  } catch (e) {
-    // in case localized string cannot be found
-  }
-  // 9) If all else fails, use "index"
-  return "index";
+
+  return "";
 }
 
+// This is only used after the user has entered a filename.
 function validateFileName(aFileName) {
-  let processed = DownloadPaths.sanitize(aFileName) || "_";
+  let processed =
+    DownloadPaths.sanitize(aFileName, {
+      compressWhitespaces: false,
+      allowInvalidFilenames: true,
+    }) || "_";
   if (AppConstants.platform == "android") {
     // If a large part of the filename has been sanitized, then we
     // will use a default filename instead
     if (processed.replace(/_/g, "").length <= processed.length / 2) {
       // We purposefully do not use a localized default filename,
       // which we could have done using
-      // ContentAreaUtils.stringBundle.GetStringFromName("DefaultSaveFileName")
+      // ContentAreaUtils.stringBundle.GetStringFromName("UntitledSaveFileName")
       // since it may contain invalid characters.
       var original = processed;
       processed = "download";
@@ -1287,80 +1117,9 @@ function validateFileName(aFileName) {
   return processed;
 }
 
-function getNormalizedLeafName(aFile, aDefaultExtension) {
-  if (!aDefaultExtension) {
-    return aFile;
-  }
-
-  if (AppConstants.platform == "win") {
-    // Remove trailing dots and spaces on windows
-    aFile = aFile.replace(/[\s.]+$/, "");
-  }
-
-  // Remove leading dots
-  aFile = aFile.replace(/^\.+/, "");
-
-  // Fix up the file name we're saving to to include the default extension
-  var i = aFile.lastIndexOf(".");
-  if (aFile.substr(i + 1) != aDefaultExtension) {
-    return aFile + "." + aDefaultExtension;
-  }
-
-  return aFile;
-}
-
-function getDefaultExtension(aFilename, aURI, aContentType) {
-  if (
-    aContentType == "text/plain" ||
-    aContentType == "application/octet-stream" ||
-    aURI.scheme == "ftp"
-  ) {
-    return "";
-  } // temporary fix for bug 120327
-
-  // First try the extension from the filename
-  var url = Cc["@mozilla.org/network/standard-url-mutator;1"]
-    .createInstance(Ci.nsIURIMutator)
-    .setSpec("http://example.com") // construct the URL
-    .setFilePath(aFilename)
-    .finalize()
-    .QueryInterface(Ci.nsIURL);
-
-  var ext = url.fileExtension;
-
-  // This mirrors some code in nsExternalHelperAppService::DoContent
-  // Use the filename first and then the URI if that fails
-
-  var mimeInfo = getMIMEInfoForType(aContentType, ext);
-
-  if (ext && mimeInfo && mimeInfo.extensionExists(ext)) {
-    return ext;
-  }
-
-  // Well, that failed.  Now try the extension from the URI
-  var urlext;
-  try {
-    url = aURI.QueryInterface(Ci.nsIURL);
-    urlext = url.fileExtension;
-  } catch (e) {}
-
-  if (urlext && mimeInfo && mimeInfo.extensionExists(urlext)) {
-    return urlext;
-  }
-  try {
-    if (mimeInfo) {
-      return mimeInfo.primaryExtension;
-    }
-  } catch (e) {}
-  // Fall back on the extensions in the filename and URI for lack
-  // of anything better.
-  return ext || urlext;
-}
-
 function GetSaveModeForContentType(aContentType, aDocument) {
   // We can only save a complete page if we have a loaded document,
-  // and it's not a CPOW -- nsWebBrowserPersist needs a real document.
-  if (!aDocument || Cu.isCrossProcessWrapper(aDocument)) {
+  if (!aDocument) {
     return SAVEMODE_FILEONLY;
   }
 
@@ -1401,17 +1160,18 @@ function getCharsetforSave(aDocument) {
  * WARNING: Please note that openURL() does not perform any content security checks!!!
  */
 function openURL(aURL) {
-  var uri = makeURI(aURL);
+  var uri = aURL instanceof Ci.nsIURI ? aURL : makeURI(aURL);
 
   var protocolSvc = Cc[
     "@mozilla.org/uriloader/external-protocol-service;1"
   ].getService(Ci.nsIExternalProtocolService);
 
+  let recentWindow = Services.wm.getMostRecentWindow("navigator:browser");
+
   if (!protocolSvc.isExposedProtocol(uri.scheme)) {
     // If we're not a browser, use the external protocol service to load the URI.
-    protocolSvc.loadURI(uri);
+    protocolSvc.loadURI(uri, recentWindow?.document.contentPrincipal);
   } else {
-    var recentWindow = Services.wm.getMostRecentWindow("navigator:browser");
     if (recentWindow) {
       recentWindow.openWebLinkIn(uri.spec, "tab", {
         triggeringPrincipal: recentWindow.document.contentPrincipal,
@@ -1439,9 +1199,6 @@ function openURL(aURL) {
     loadgroup.groupObserver = loadListener;
 
     var uriListener = {
-      onStartURIOpen(uri) {
-        return false;
-      },
       doContent(ctype, preferred, request, handler) {
         return false;
       },
@@ -1460,7 +1217,7 @@ function openURL(aURL) {
         if (iid.equals(Ci.nsILoadGroup)) {
           return loadgroup;
         }
-        throw Cr.NS_ERROR_NO_INTERFACE;
+        throw Components.Exception("", Cr.NS_ERROR_NO_INTERFACE);
       },
     };
 
