@@ -219,7 +219,7 @@ class MinidumpWriter {
   bool Dump() {
     // A minidump file contains a number of tagged streams. This is the number
     // of stream which we write.
-    unsigned kNumWriters = 13;
+    unsigned kNumWriters = 14;
 
     TypedMDRVA<MDRawDirectory> dir(&minidump_writer_);
     {
@@ -248,6 +248,10 @@ class MinidumpWriter {
       return false;
     dir.CopyIndex(dir_index++, &dirent);
 
+    if (!WriteThreadNamesStream(&dirent))
+      return false;
+    dir.CopyIndex(dir_index++, &dirent);
+
     if (!WriteMappings(&dirent))
       return false;
     dir.CopyIndex(dir_index++, &dirent);
@@ -278,8 +282,10 @@ class MinidumpWriter {
     dir.CopyIndex(dir_index++, &dirent);
 
     dirent.stream_type = MD_LINUX_LSB_RELEASE;
-    if (!WriteFile(&dirent.location, "/etc/lsb-release"))
+    if (!WriteFile(&dirent.location, "/etc/lsb-release") &&
+        !WriteFile(&dirent.location, "/etc/os-release")) {
       NullifyDirectoryEntry(&dirent);
+    }
     dir.CopyIndex(dir_index++, &dirent);
 
     dirent.stream_type = MD_LINUX_CMD_LINE;
@@ -511,6 +517,52 @@ class MinidumpWriter {
     return true;
   }
 
+  bool WriteThreadName(pid_t tid, char* name, MDRawThreadName *thread_name) {
+    MDLocationDescriptor string_location;
+
+    if (!minidump_writer_.WriteString(name, 0, &string_location))
+      return false;
+
+    thread_name->thread_id = tid;
+    thread_name->rva_of_thread_name = string_location.rva;
+    return true;
+  }
+
+  // Write the threads' names.
+  bool WriteThreadNamesStream(MDRawDirectory* thread_names_stream) {
+    TypedMDRVA<MDRawThreadNamesList> list(&minidump_writer_);
+    const unsigned num_threads = dumper_->threads().size();
+
+    if (!list.AllocateObjectAndArray(num_threads, sizeof(MDRawThreadName))) {
+      return false;
+    }
+
+    thread_names_stream->stream_type = MD_THREAD_NAMES_STREAM;
+    thread_names_stream->location = list.location();
+    list.get()->number_of_thread_names = num_threads;
+
+    MDRawThreadName thread_name;
+    int thread_idx = 0;
+
+    for (unsigned int i = 0; i < num_threads; ++i) {
+      const pid_t tid = dumper_->threads()[i];
+      // This is a constant from the Linux kernel, documented in man 5 proc.
+      // The comm entries in /proc are no longer than this.
+      static const size_t TASK_COMM_LEN = 16;
+      char name[TASK_COMM_LEN];
+      memset(&thread_name, 0, sizeof(MDRawThreadName));
+
+      if (dumper_->GetThreadNameByIndex(i, name, sizeof(name))) {
+        if (WriteThreadName(tid, name, &thread_name)) {
+          list.CopyIndexAfterObject(thread_idx++, &thread_name,
+                                  sizeof(MDRawThreadName));
+        }
+      }
+    }
+
+    return true;
+  }
+
   // Write application-provided memory regions.
   bool WriteAppMemory() {
     for (AppMemoryList::const_iterator iter = app_memory_list_.begin();
@@ -642,10 +694,12 @@ class MinidumpWriter {
                               identifier->end());
     } else {
       // Note: ElfFileIdentifierForMapping() can manipulate the |mapping.name|.
-      dumper_->ElfFileIdentifierForMapping(mapping,
-                                           member,
-                                           mapping_id,
-                                           identifier_bytes);
+      if (!dumper_->ElfFileIdentifierForMapping(mapping,
+                                                member,
+                                                mapping_id,
+                                                identifier_bytes)) {
+        identifier_bytes.clear();
+      }
     }
 
     if (!identifier_bytes.empty()) {
@@ -753,7 +807,7 @@ class MinidumpWriter {
     ElfW(Addr) dyn_addr = 0;
     for (; phnum >= 0; phnum--, phdr++) {
       ElfW(Phdr) ph;
-      if (!dumper_->CopyFromProcess(&ph, GetCrashThread(), phdr, sizeof(ph)))
+      if (!dumper_->CopyFromProcess(&ph, dumper_->pid(), phdr, sizeof(ph)))
         return false;
 
       // Adjust base address with the virtual address of the PT_LOAD segment
@@ -779,8 +833,7 @@ class MinidumpWriter {
     for (int i = 0; ; ++i) {
       ElfW(Dyn) dyn;
       dynamic_length += sizeof(dyn);
-      if (!dumper_->CopyFromProcess(&dyn, GetCrashThread(), dynamic + i,
-                                    sizeof(dyn))) {
+      if (!dumper_->CopyFromProcess(&dyn, dumper_->pid(), dynamic + i, sizeof(dyn))) {
         return false;
       }
 
@@ -808,13 +861,13 @@ class MinidumpWriter {
     // Count the number of loaded DSOs
     int dso_count = 0;
     struct r_debug debug_entry;
-    if (!dumper_->CopyFromProcess(&debug_entry, GetCrashThread(), r_debug,
+    if (!dumper_->CopyFromProcess(&debug_entry, dumper_->pid(), r_debug,
                                   sizeof(debug_entry))) {
       return false;
     }
     for (struct link_map* ptr = debug_entry.r_map; ptr; ) {
       struct link_map map;
-      if (!dumper_->CopyFromProcess(&map, GetCrashThread(), ptr, sizeof(map)))
+      if (!dumper_->CopyFromProcess(&map, dumper_->pid(), ptr, sizeof(map)))
         return false;
 
       ptr = map.l_next;
@@ -834,13 +887,13 @@ class MinidumpWriter {
       // Iterate over DSOs and write their information to mini dump
       for (struct link_map* ptr = debug_entry.r_map; ptr; ) {
         struct link_map map;
-        if (!dumper_->CopyFromProcess(&map, GetCrashThread(), ptr, sizeof(map)))
+        if (!dumper_->CopyFromProcess(&map, dumper_->pid(), ptr, sizeof(map)))
           return  false;
 
         ptr = map.l_next;
         char filename[257] = { 0 };
         if (map.l_name) {
-          dumper_->CopyFromProcess(filename, GetCrashThread(), map.l_name,
+          dumper_->CopyFromProcess(filename, dumper_->pid(), map.l_name,
                                    sizeof(filename) - 1);
         }
         MDLocationDescriptor location;
@@ -873,7 +926,7 @@ class MinidumpWriter {
     // The passed-in size to the constructor (above) is only a hint.
     // Must call .resize() to do actual initialization of the elements.
     dso_debug_data.resize(dynamic_length);
-    dumper_->CopyFromProcess(&dso_debug_data[0], GetCrashThread(), dynamic,
+    dumper_->CopyFromProcess(&dso_debug_data[0], dumper_->pid(), dynamic,
                              dynamic_length);
     debug.CopyIndexAfterObject(0, &dso_debug_data[0], dynamic_length);
 
@@ -1424,8 +1477,10 @@ bool WriteMinidump(const char* minidump_path, pid_t process,
   // MinidumpWriter will set crash address
   dumper.set_crash_signal(MD_EXCEPTION_CODE_LIN_DUMP_REQUESTED);
   dumper.set_crash_thread(process_blamed_thread);
-  MinidumpWriter writer(minidump_path, -1, NULL, MappingList(),
-                        AppMemoryList(), false, 0, false, &dumper);
+  MappingList mapping_list;
+  AppMemoryList app_memory_list;
+  MinidumpWriter writer(minidump_path, -1, NULL, mapping_list,
+                        app_memory_list, false, 0, false, &dumper);
   if (!writer.Init())
     return false;
   return writer.Dump();
