@@ -9,11 +9,7 @@ extern crate app_services_logger;
 extern crate audioipc2_client;
 #[cfg(feature = "cubeb-remoting")]
 extern crate audioipc2_server;
-#[cfg(feature = "cubeb-remoting")]
-extern crate audioipc_client;
-#[cfg(feature = "cubeb-remoting")]
-extern crate audioipc_server;
-extern crate authenticator;
+extern crate authrs_bridge;
 #[cfg(feature = "bitsdownload")]
 extern crate bitsdownload;
 #[cfg(feature = "moz_places")]
@@ -32,6 +28,7 @@ extern crate fog_control;
 extern crate gecko_profiler;
 extern crate gkrust_utils;
 extern crate http_sfv;
+extern crate jog;
 extern crate jsrust_shared;
 extern crate kvstore;
 extern crate mapped_hyph;
@@ -49,20 +46,28 @@ extern crate static_prefs;
 extern crate storage;
 extern crate webrender_bindings;
 extern crate xpcom;
-#[cfg(feature = "new_xulstore")]
-extern crate xulstore;
 
 extern crate audio_thread_priority;
 
 #[cfg(not(target_os = "android"))]
 extern crate webext_storage_bridge;
 
+#[cfg(not(target_os = "android"))]
+extern crate tabs;
+
+#[cfg(not(target_os = "android"))]
+mod reexport_tabs {
+    tabs::uniffi_reexport_scaffolding!();
+}
+
 #[cfg(feature = "webrtc")]
 extern crate mdns_service;
 extern crate neqo_glue;
 extern crate wgpu_bindings;
 
+extern crate aa_stroke;
 extern crate qcms;
+extern crate wpf_gpu_raster;
 
 extern crate unic_langid;
 extern crate unic_langid_ffi;
@@ -73,7 +78,7 @@ extern crate fluent_langneg_ffi;
 extern crate fluent;
 extern crate fluent_ffi;
 
-extern crate regex_ffi;
+extern crate rure;
 
 extern crate fluent_fallback;
 extern crate l10nregistry_ffi;
@@ -95,6 +100,33 @@ extern crate detect_win32k_conflicts;
 
 extern crate origin_trials_ffi;
 
+extern crate dap_ffi;
+
+extern crate data_encoding_ffi;
+
+extern crate binary_http;
+extern crate oblivious_http;
+
+extern crate mime_guess_ffi;
+
+#[cfg(feature = "uniffi_fixtures")]
+mod uniffi_fixtures {
+    extern crate arithmetical;
+    extern crate uniffi_geometry;
+    extern crate uniffi_rondpoint;
+    extern crate uniffi_sprites;
+    extern crate uniffi_todolist;
+
+    arithmetical::uniffi_reexport_scaffolding!();
+    uniffi_fixture_callbacks::uniffi_reexport_scaffolding!();
+    uniffi_custom_types::uniffi_reexport_scaffolding!();
+    uniffi_fixture_external_types::uniffi_reexport_scaffolding!();
+    uniffi_geometry::uniffi_reexport_scaffolding!();
+    uniffi_rondpoint::uniffi_reexport_scaffolding!();
+    uniffi_sprites::uniffi_reexport_scaffolding!();
+    uniffi_todolist::uniffi_reexport_scaffolding!();
+}
+
 extern crate log;
 use log::info;
 
@@ -113,15 +145,13 @@ pub extern "C" fn GkRust_Shutdown() {}
 
 /// Used to implement `nsIDebug2::RustPanic` for testing purposes.
 #[no_mangle]
-pub extern "C" fn intentional_panic(message: *const c_char) {
-    panic!("{}", unsafe { CStr::from_ptr(message) }.to_string_lossy());
+pub unsafe extern "C" fn intentional_panic(message: *const c_char) {
+    panic!("{}", CStr::from_ptr(message).to_string_lossy());
 }
 
 /// Used to implement `nsIDebug2::rustLog` for testing purposes.
 #[no_mangle]
-pub extern "C" fn debug_log(target: *const c_char, message: *const c_char) {
-    unsafe {
-        // NOTE: The `info!` log macro is used here because we have the `release_max_level_info` feature set.
-        info!(target: CStr::from_ptr(target).to_str().unwrap(), "{}", CStr::from_ptr(message).to_str().unwrap());
-    }
+pub unsafe extern "C" fn debug_log(target: *const c_char, message: *const c_char) {
+    // NOTE: The `info!` log macro is used here because we have the `release_max_level_info` feature set.
+    info!(target: CStr::from_ptr(target).to_str().unwrap(), "{}", CStr::from_ptr(message).to_str().unwrap());
 }
