@@ -5,15 +5,13 @@
 
 /* eslint no-unused-vars: ["error", {vars: "local", args: "none"}] */
 
-var { NetUtil } = ChromeUtils.import("resource://gre/modules/NetUtil.jsm");
-const { TelemetryTestUtils } = ChromeUtils.import(
-  "resource://testing-common/TelemetryTestUtils.jsm"
+const { TelemetryTestUtils } = ChromeUtils.importESModule(
+  "resource://testing-common/TelemetryTestUtils.sys.mjs"
 );
 
-let { AddonManager, AddonManagerPrivate } = ChromeUtils.import(
-  "resource://gre/modules/AddonManager.jsm"
+let { AddonManagerPrivate } = ChromeUtils.importESModule(
+  "resource://gre/modules/AddonManager.sys.mjs"
 );
-let { Log } = ChromeUtils.import("resource://gre/modules/Log.jsm");
 
 var pathParts = gTestPath.split("/");
 // Drop the test filename
@@ -37,7 +35,7 @@ const PREF_LOGGING_ENABLED = "extensions.logging.enabled";
 const PREF_STRICT_COMPAT = "extensions.strictCompatibility";
 
 var PREF_CHECK_COMPATIBILITY;
-(function() {
+(function () {
   var channel = Services.prefs.getCharPref("app.update.channel", "default");
   if (
     channel != "aurora" &&
@@ -125,7 +123,7 @@ function enableBackgroundUpdateTimer() {
   );
 }
 
-registerCleanupFunction(function() {
+registerCleanupFunction(function () {
   // Restore prefs
   for (let pref of gRestorePrefs) {
     if (pref.type == "clear") {
@@ -206,7 +204,7 @@ function run_next_test() {
   executeSoon(() => log_exceptions(test));
 }
 
-var get_tooltip_info = async function(addonEl, managerWindow) {
+var get_tooltip_info = async function (addonEl, managerWindow) {
   // Extract from title attribute.
   const { addon } = addonEl;
   const name = addon.name;
@@ -301,7 +299,7 @@ async function wait_for_view_load(
 
     aManagerWindow.document.addEventListener(
       "view-loaded",
-      function() {
+      function () {
         resolve(aManagerWindow);
       },
       { once: true }
@@ -422,8 +420,8 @@ function wait_for_window_open(aCallback) {
         let domwindow = aXulWin.docShell.domWindow;
         domwindow.addEventListener(
           "load",
-          function() {
-            executeSoon(function() {
+          function () {
+            executeSoon(function () {
               resolve(domwindow);
             });
           },
@@ -589,16 +587,14 @@ CategoryUtilities.prototype = {
 
 // Returns a promise that will resolve when the certificate error override has been added, or reject
 // if there is some failure.
-function addCertOverride(host, bits) {
+function addCertOverride(host) {
   return new Promise((resolve, reject) => {
     let req = new XMLHttpRequest();
     req.open("GET", "https://" + host + "/");
     req.onload = reject;
     req.onerror = () => {
       if (req.channel && req.channel.securityInfo) {
-        let securityInfo = req.channel.securityInfo.QueryInterface(
-          Ci.nsITransportSecurityInfo
-        );
+        let securityInfo = req.channel.securityInfo;
         if (securityInfo.serverCert) {
           let cos = Cc["@mozilla.org/security/certoverride;1"].getService(
             Ci.nsICertOverrideService
@@ -608,7 +604,6 @@ function addCertOverride(host, bits) {
             -1,
             {},
             securityInfo.serverCert,
-            bits,
             false
           );
           resolve();
@@ -624,34 +619,22 @@ function addCertOverride(host, bits) {
 // Returns a promise that will resolve when the necessary certificate overrides have been added.
 function addCertOverrides() {
   return Promise.all([
-    addCertOverride(
-      "nocert.example.com",
-      Ci.nsICertOverrideService.ERROR_MISMATCH
-    ),
-    addCertOverride(
-      "self-signed.example.com",
-      Ci.nsICertOverrideService.ERROR_UNTRUSTED
-    ),
-    addCertOverride(
-      "untrusted.example.com",
-      Ci.nsICertOverrideService.ERROR_UNTRUSTED
-    ),
-    addCertOverride(
-      "expired.example.com",
-      Ci.nsICertOverrideService.ERROR_TIME
-    ),
+    addCertOverride("nocert.example.com"),
+    addCertOverride("self-signed.example.com"),
+    addCertOverride("untrusted.example.com"),
+    addCertOverride("expired.example.com"),
   ]);
 }
 
 /** *** Mock Provider *****/
 
-function MockProvider() {
+function MockProvider(addonTypes) {
   this.addons = [];
   this.installs = [];
-  this.addonTypes = ["extension"];
+  this.addonTypes = addonTypes ?? ["extension"];
 
   var self = this;
-  registerCleanupFunction(function() {
+  registerCleanupFunction(function () {
     if (self.started) {
       self.unregister();
     }
@@ -913,7 +896,7 @@ MockProvider.prototype = {
   async getAddonsByTypes(aTypes) {
     await this.queryDelayPromise;
 
-    var addons = this.addons.filter(function(aAddon) {
+    var addons = this.addons.filter(function (aAddon) {
       if (aTypes && !!aTypes.length && !aTypes.includes(aAddon.type)) {
         return false;
       }
@@ -931,7 +914,7 @@ MockProvider.prototype = {
   async getInstallsByTypes(aTypes) {
     await this.queryDelayPromise;
 
-    var installs = this.installs.filter(function(aInstall) {
+    var installs = this.installs.filter(function (aInstall) {
       // Appear to have actually removed cancelled installs from the provider
       if (aInstall.state == AddonManager.STATE_CANCELLED) {
         return false;
@@ -1406,7 +1389,7 @@ MockInstall.prototype = {
 
 function waitForCondition(condition, nextTest, errorMsg) {
   let tries = 0;
-  let interval = setInterval(function() {
+  let interval = setInterval(function () {
     if (tries >= 30) {
       ok(false, errorMsg);
       moveOn();
@@ -1423,7 +1406,7 @@ function waitForCondition(condition, nextTest, errorMsg) {
     }
     tries++;
   }, 100);
-  let moveOn = function() {
+  let moveOn = function () {
     clearInterval(interval);
     nextTest();
   };
@@ -1478,8 +1461,8 @@ function waitAppMenuNotificationShown(
   accept = false,
   win = window
 ) {
-  const { AppMenuNotifications } = ChromeUtils.import(
-    "resource://gre/modules/AppMenuNotifications.jsm"
+  const { AppMenuNotifications } = ChromeUtils.importESModule(
+    "resource://gre/modules/AppMenuNotifications.sys.mjs"
   );
   return new Promise(resolve => {
     let { document, PanelUI } = win;
@@ -1529,21 +1512,6 @@ function acceptAppMenuNotificationWhenShown(id, addonId) {
   return waitAppMenuNotificationShown(id, addonId, true);
 }
 
-const ABOUT_ADDONS_METHODS = new Set(["action", "view", "link"]);
-function assertAboutAddonsTelemetryEvents(events, filters = {}) {
-  TelemetryTestUtils.assertEvents(events, {
-    category: "addonsManager",
-    method: actual =>
-      filters.methods
-        ? filters.methods.includes(actual)
-        : ABOUT_ADDONS_METHODS.has(actual),
-    object: actual =>
-      filters.objects
-        ? filters.objects.includes(actual)
-        : actual === "aboutAddons",
-  });
-}
-
 /* HTML view helpers */
 async function loadInitialView(type, opts) {
   if (type) {
@@ -1581,13 +1549,8 @@ async function loadInitialView(type, opts) {
   return win;
 }
 
-function getSection(doc, type) {
-  const SECTION_INDEXES = {
-    enabled: 0,
-    disabled: 1,
-    colorway: 2,
-  };
-  return doc.querySelector(`section[section="${SECTION_INDEXES[type]}"]`);
+function getSection(doc, className) {
+  return doc.querySelector(`section.${className}`);
 }
 
 function waitForViewLoad(win) {
