@@ -8,7 +8,7 @@ function test() {
 
   gBrowser.selectedTab = BrowserTestUtils.addTab(gBrowser, "about:blank");
   BrowserTestUtils.browserLoaded(gBrowser.selectedBrowser).then(() => {
-    BrowserTestUtils.loadURI(gBrowser, TESTROOT + "amosigned.xpi");
+    BrowserTestUtils.loadURIString(gBrowser, TESTROOT + "amosigned.xpi");
   });
 }
 
@@ -18,7 +18,7 @@ function confirm_install(panel) {
 }
 
 function install_ended(install, addon) {
-  install.cancel();
+  return addon.uninstall();
 }
 
 function finish_test(count) {
